@@ -6,6 +6,10 @@ var sessionID;
 var auth;
 var itemid;
 var templid;
+var logindata;
+var attachlist = [];
+var uploadlist = [];
+var savedata;
 
 
 try {// Load root from config file.  Error if missing or unable to determine api address
@@ -87,7 +91,7 @@ function login() {
         password = $('#Password').val().trim();
     }
     $('#authorizing').show();
-    var logindata = {
+    logindata = {
             username : username,
             password : password
     }
@@ -109,32 +113,31 @@ function login() {
         if (data.auth === "Bad Password" || data.auth === "No Key") {
             //#. Bad username or Password at login
             $('#ErrorBox').html(i18n.t("Bad username / Password"));
-	    $('#ErrorBox').show();
+            $('#ErrorBox').show();
             $('#Username').focus();
             $('#Username').val('');
             $('#Password').val('');
             $('#authorizing').hide();
+            return;
         }
         if (data.auth === "lockout") {
             $('#ErrorBox').html(i18n.t("Account locked out try later"));
             $('#ErrorBox').show();
-	    $('#Username').hide();
+            $('#Username').hide();
             $('#Password').hide();
             $('#LoginButton').hide();
             $('#authorizing').hide();
+            return;
         }
         if (data.auth.length > 20) {
-            userid = data.userid;
-            cid = data.cid;
-            sessionID = data.sessionID;
-            auth = data.auth;
-            var extrapass = getURLParameter("extrapass");
-            if (extrapass !== null) extrapass = extrapass.trim();
-            if (extrapass === 'true') {
-            	getExtraPass(data);
-            } else {
-            	openreader(data, "");
-            }
+            okLogin(data);
+            return;
+        }
+        if (data.auth === "PIN") {
+            $('#authorizing').hide();
+            $('#pin').show();
+            $('#pinbox').focus();
+            return;
         }
 
     })
@@ -144,6 +147,52 @@ function login() {
     });
 }
 
+function okLogin(data) {
+    userid = data.userid;
+    cid = data.cid;
+    sessionID = data.sessionID;
+    auth = data.auth;
+    var extrapass = getURLParameter("extrapass");
+    if (extrapass !== null) extrapass = extrapass.trim();
+    if (extrapass === 'true') {
+        getExtraPass(data);
+    } else {
+        openreader(data, "");
+    }
+    logindata = null;
+}
+
+function loginPin() {
+    logindata.pin = $('#pinbox').val();
+    $.post(root + "oxguard/login?action=guest", JSON.stringify(logindata), function (data) {
+        try {
+            log("loging data: ", data);
+            if (data.auth.length > 20) {
+                okLogin(data);
+                return;
+            }
+            if (data.auth === "Bad") {
+                $('#PinErrorBox').html(i18n.t("Bad PIN")).show();
+                $('#pinbox').val('').focus();
+                return;
+            }
+            if (data.auth === "lockout") {
+                $('#PinErrorBox').html(i18n.t("Account locked out try later")).show();
+                $('#pinbox').val('').hide();
+                $('#PinButton').hide();
+                return;
+            }
+        } catch (e) {
+            $('#PinErrorBox').html(i18n.t("Error checking PIN")).show();
+            log(e);
+        }
+    })
+    .fail(function () {
+        $('#PinErrorBox').html(i18n.t("Error checking PIN")).show();
+        return;
+    });
+}
+
 function showChangepass() {
 	view('NewPassword');
 }
@@ -190,26 +239,30 @@ function changePass() {
     });
 }
 
+function showChangePassError (text) {
+    $('#firstChangeError').html(text);
+    $('#firstChangeError').show();
+    if ($(window.top).height() < 800) // If small, scroll to bottom
+        $('.passchange').animate ({ scrollTop: $('#firstChangeError').offset().top }, 1000);
+}
+
 //Change the oxguard password
 function firstChangePass() {
     var pass1 = $('#change1').val();
     if (pass1.length < 8) {
         $('#change1').focus();
-        $('#firstChangeError').html(i18n.t("Password must be at least 8 characters long"));
-        $('#firstChangeError').show();
+        showChangePassError(i18n.t("Password must be at least 8 characters long"));
         return;
     }
     if (pass1 !== $('#change2').val()) {
         $('#change1').focus();
-        $('#firstChangeError').html(i18n.t("Passwords do not match"));
-        $('#firstChangeError').show();
+        showChangePassError(i18n.t("Passwords do not match"));
         return;
     }
     var answer = $('#answer').val();
     if (answer.length < 3) {
         $('#answer').focus();
-        $('#firstChangeError').html(i18n.t("Please provide an answer for the security question"));
-        $('#firstChangeError').show();
+        showChangePassError(i18n.t("Please provide an answer for the security question"));
         return;
     }
     var password = $('#Password').val();
@@ -241,11 +294,12 @@ function firstChangePass() {
                 $('#firstChangeError').hide;
                 $('#firstChangePassDiv').hide();
             },1000);
+            $('#Reader').css("position", "inherit");
         }
 
     })
     .fail(function () {
-        $('#firstChangeError').html(i18n.t("Failed to change password"));
+        showChangePassError(i18n.t("Failed to change password"));
         $('#firstChangeError').removeClass('alert-success').addClass('alert-danger').show();
     });
 }
@@ -267,8 +321,8 @@ function getExtraPass (data) {
 }
 
 // Open the email for reading.  Populate with decoded data
-function openreader(data, extra) {
-    var lastMod = data.lastMod;
+function openreader(ldata, extra) {
+    var lastMod = ldata.lastMod;
     view();
     $('#decoding').show();
     itemid = getURLParameter("email");
@@ -287,16 +341,16 @@ function openreader(data, extra) {
         }
         var toList = getTo(data),
             ccList = getCC(data);
-        $('#sent').html(i18n.t("Sent") + '<span class=\"oxlabelcontent\">' + getSent(data) + '</span>');
+        $('#sent').html('<td class="tdname">' + i18n.t("Sent") + '</td><td><span class=\"oxlabelcontent\">' + prepHTML(getSent(data)) + '</span></td>');
         if (toList.length > 2) {
-            $('#tolist').html(i18n.t("To") + '<span class=\"oxlabelcontent\">' + toList + '</span>').show();
+            $('#tolist').html('<td class="tdname">' + i18n.t("To") + '</td><td><span class=\"oxlabelcontent\">' + prepHTML(toList) + '</span></td>').show();
         }
         log("cc: ", ccList);
         if (ccList.length > 2) {
-            $('#cclist').html(i18n.t("Cc") +'<span class=\"oxlabelcontent\">' + ccList + '</span>').show();
+            $('#cclist').html('<td class="tdname">' + i18n.t("Cc") +'</td><td><span class=\"oxlabelcontent\">' + prepHTML(ccList) + '</span></td>').show();
         }
 
-        $('#fromlist').html(i18n.t("From") +'<span class=\"oxlabelcontent\">' + getFrom(data) + '</span>');
+        $('#fromlist').html('<td class="tdname">' + i18n.t("From") +'</td><td><span class=\"oxlabelcontent\">' + prepHTML(getFrom(data)) + '</span></td>');
         $('#subject').html(getSubject(data));
         $('#attdiv').remove();
         var readercont = $('#reader').contents().find('html');
@@ -316,13 +370,38 @@ function openreader(data, extra) {
         $('#replyall-bottom').unbind().click(function() {
                 replyall(data);
         });
+        // Some squeeze for mobile
+        if (window.screen.height < 800) {
+            log('mobile detected');
+            $('.padding').hide();
+            $('.attachments').css('padding', '0');
+        }
         log("last mod: ", lastMod);
         if (lastMod === 'null') {
             log('for changepass');
             window.setTimeout(function () {
+                $('#Reader').css("position", "fixed");
+                if((navigator.userAgent.match(/iPhone/i)) || (navigator.userAgent.match(/iPod/i))) {
+                    $('.passchange').css('height', '100%').css('width', '100%').css('margin-top', '0%')
+                    .bind('touchstart', function (e) {
+                        console.log(e.target.type);
+                        if (e.target.type !== 'password' && e.target.type !== 'text') {
+                            document.activeElement.blur();
+                        }
+                    });
+                }
                 $('#firstChangePassDiv').show();
             },2000);
         }
+        // Make all name columns same size.  Check for largest
+        window.setTimeout(function() {
+            var max = 0;
+            $('.tdname').each(function(v, e) { 
+                var w = $(e).width();
+                if (w > max) max = w;
+            });
+            $('.tdname').width(max + 2);
+        }, 100);
 
     })
     .fail(function (data) {
@@ -334,14 +413,15 @@ function openreader(data, extra) {
             break;
         case "not found":
             view("NotFound");
+            savedata = ldata;
             break;
         case "retracted":
-            fail("This email is no longer available");
+            fail(i18n.t("This email is no longer available"));
             break;
         case "expired":
-            fail("This email is no longer available");
+            fail(i18n.t("This email is no longer available"));
             break;
-        case "default":
+        default:
             fail();
             break;
         }
@@ -355,7 +435,7 @@ function fail(alerttype)
     if (alerttype === undefined) {
         alert(i18n.t("An error occurred"));
     } else {
-        alert(i18n.t(alerttype));
+        alert(alerttype);
     }
     $('#authorizing').hide();
     $('#decoding').hide();
@@ -397,7 +477,7 @@ function getTinyLanguage () {
 
 // Add surrounding <> for emails
 function surround(d) {
-    return (' &lt;' + d + '&gt;');
+    return (' <' + d + '>');
 }
 
 function getList(recips) {
@@ -440,6 +520,10 @@ function getSubject(data) {
     return (data.subject);
 }
 
+function prepHTML (data) {
+    return $('<div/>').text(data).html();
+}
+
 function getSent(data) {
   //  var sent = new Date(Date.parse(data.sent));
   //  return (sent.toLocaleString());
@@ -465,13 +549,16 @@ function getContent(data) {
 // Generate attachments.
 function getAttachments(data, item, auth) {
     var attachmentdiv = $('<div id="attdiv"/>');
+    var atttable = $('<table>');
+    var row1 = $('<tr>');
     var attachments = data.attachments;
     var attcount = 0;
+    var attd = $('<td>');
     for (var z = 0; z < attachments.length; z++) {
         var attach = attachments[z];
         if (attach.disp === 'attachment') {//If it is an attachment, we are going to display
             if (attcount === 0) {
-                attachmentdiv.append('<span class="io-ox-label">' + i18n.t("Attachment") + '&nbsp&nbsp</span>');
+                row1.append('<td class="io-ox-label" id="attlabel" style="vertical-align:top; padding-top:5px;">' + i18n.t("Attachment") + '</td>');
             }
             attcount++;
             // Create the link for that attachment
@@ -483,19 +570,30 @@ function getAttachments(data, item, auth) {
             '&lang=' + language +
             '&attname=' + encodeURIComponent(attach.filename);
             var link = root + 'oxguard/guest?action=getattach' + params;
-            var atlink = $('<a/>', {
+            var atlink = $('<a>', {
                 "class": "attachment-link",
                 text: attach.filename.replace('.grd2','').replace('.grd',''),
-                "link": link
+                "link": link,
+                "href" : "#"
             }).css({
                 'padding-right': '7px'
             }).click(function() {
                 window.open($(this).attr("link") + "&auth=" + encodeURIComponent(getauth()));
             });
-            attachmentdiv.append(atlink);
+            attd.append($('<div class="attachment-link">').append(atlink));
         }
     }
-    return (attachmentdiv);
+    row1.append(attd);
+    // Resize
+    window.setTimeout(function () {
+        try {
+            var size = window.screen.width - $('#attlabel').width() - 50;
+            $('.attachment-link').css('max-width', size);
+        } catch (e) {
+            console.log(e);
+        }
+    }, 100);
+    return (attachmentdiv.append(atttable.append(row1)));
 }
 
 function getauth () {
@@ -503,6 +601,7 @@ function getauth () {
 }
 
 function getReplyData (data) {
+    
     var replyTo = JSON.parse(JSON.stringify(data.from));
     var replyAll = JSON.parse(JSON.stringify(data.from));
     var thisEmail = $('#Username').val();
@@ -529,6 +628,61 @@ function getReplyData (data) {
     }
 }
 
+function changeAttachments(attachments) {
+    // Push to attachment list
+    for (var i=0; i<attachments.files.length; i++) {
+        attachlist.push(attachments.files[i]);
+    }
+    $(attachments).val('');  // Clear file list
+    drawAttachments();
+    
+}
+
+
+
+
+// Draw attachments for reply
+function drawAttachments () {
+    var list = '';
+    $('.deleteAttach').unbind();
+    for (var i=0; i< attachlist.length; i++) {
+        if (i === 0) {
+            list = createLink(attachlist[i]);
+        } else {
+            list = list + createLink(attachlist[i]);
+        }
+    }
+
+    $('#attachmentList').html($('<ul>').append(list));
+    $('.deleteAttach').click(function (e) {
+        deleteAttachment($(e.target).parent().attr('data'));
+        drawAttachments ();
+    });
+}
+
+function createLink (file) {
+    if (file === undefined) return('');
+    var name = file.name;
+    var div = $('<ul>');
+    var span = $('<li>');
+    var deletelink = $('<a href="#" id="delete-' + name + '" data="' + name + '" class="deleteAttach"></a>');
+    var trash = $('<i class="fa fa-trash-o"></i>');
+    deletelink.append(trash);
+    div.append(span.append(deletelink).append(name));
+    return (div.html());
+}
+
+function deleteAttachment (name) {
+    for (var i =0; i < attachlist.length; i++) {
+        if (attachlist[i] !== undefined) {
+            if (attachlist[i].name.trim() === name.trim()) {
+                delete attachlist[i];
+                return;
+            }
+        }
+    }
+}
+
 // Send the reply
 function sendreply(data, all) {
     var replydata = getReplyData (data);
@@ -541,13 +695,24 @@ function sendreply(data, all) {
 	var from = replydata.from;
 	//#. For the prefix of the subject reply I.E. Re: The message you sent
 	var subject = i18n.t("re") + ': ' + data.subject;
-	var content = tinyMCE.activeEditor.getContent();
+	var format = $('[name="format"]:checked').val();
+	var content;
+	if (format === 'html') {
+	    content = tinyMCE.activeEditor.getContent();
+	}
+	else {
+	    content = $('.plaintexteditor').val();
+	} 
 
-	var mail = createJson (to, from, subject, content, data);
+	var mail = createJson (to, from, subject, content, data, format);
 	log (mail);
 	var formData = new FormData();
     formData.append('json_0', JSON.stringify(mail));
-    
+    log('attachments', attachlist);
+    for (var l = 0; l < attachlist.length; l++) {
+        if (attachlist[l] !== undefined)
+            formData.append('file' + l, attachlist[l]);
+    }
     var xhr = new XMLHttpRequest();
     var url = root + 'oxguard/mail?action=emailform' +
     '&session=' + sessionID + '&userid=' + userid + '&cid=' + cid + "&guest=true&sr=" + itemid + "&auth=" + encodeURIComponent(auth) +
@@ -558,6 +723,21 @@ function sendreply(data, all) {
         data: formData,
         processData: false,  // tell jQuery not to process the data
         contentType: false,   // tell jQuery not to set contentType
+        xhr: function() {
+            var myXhr = $.ajaxSettings.xhr();
+            if (myXhr.upload) {
+                myXhr.upload.addEventListener('progress',function(ev) {
+                    if (ev.lengthComputable) {
+                        var percentComplete = Math.floor(ev.loaded * 100 / ev.total);
+                        $('#sendingProgress').attr('aria-valuenow', percentComplete)
+                        .css('width', percentComplete + '%');
+                        log('Uploaded '+percentComplete+'%');
+                        // update UI to reflect percentUploaded
+                    }
+               }, false);
+            }
+            return myXhr;
+        },
         success: function(data){
             view("results");
             $('#sendresult').html(i18n.t("Encrypted email reply has been sent successfully"));
@@ -569,7 +749,12 @@ function sendreply(data, all) {
         error: function (XMLHttpRequest, textStatus, errorThrown) {
             log(errorThrown);
             view("results");
-            $('#sendresult').html(i18n.t("An error occurred during sending the encrypted email reply"));
+            if (XMLHttpRequest.responseText.trim() === 'not authorized') {
+                $('#sendresult').html(i18n.t("The authorization to send a reply was not valid  Did you have another reader open?  Please reload and log in again"));
+            } else {
+                $('#sendresult').html(i18n.t("An error occurred during sending the encrypted email reply"));
+            }
+            
             $('#sending').hide();
         }
       });
@@ -578,32 +763,32 @@ function sendreply(data, all) {
 }
 
 function reply(data) {
+    initReply(data);
+	$('#rto').html('<span>' + prepHTML(getFrom(data)) + '</span>').show();
+}
+
+function replyall(data) {
+    initReply (data);
     var replydata = getReplyData (data);
-	log("reply from: ", replydata.from);
-	$('#rto').html('<span>' + getFrom(data) + '</span>').show();
-	$('#rsubject').html(i18n.t("re") + ': ' + data.subject);
+    $('#rto').html('<span>' + prepHTML(getReplyAllTo(replydata)) + '</span>').show();
+}
+
+function initReply (data) {
+    $('[name="format"][value="html"]').prop("checked", true);
+    $('.plaintexteditor').val('');
+    attachlist = [];
+    drawAttachments();
+    $('#rsubject').html(i18n.t("re") + ': ' + data.subject);
     var bm = tinyMCE.activeEditor.selection.getBookmark();
     tinyMCE.activeEditor.setContent(formatReply(data));
     tinyMCE.activeEditor.selection.moveToBookmark(bm);
     tinyMCE.activeEditor.focus();
-	view('replydiv');
-	$('#SendReply').unbind().click(function() {
-		sendreply(data, false);
-	});
-}
-
-function replyall(data) {
-    var replydata = getReplyData (data);
-	$('#rto').html('<span>' + getReplyAllTo(replydata) + '</span>').show();
-	$('#rsubject').html(i18n.t("re") + ': ' + data.subject);
-	var bm = tinyMCE.activeEditor.selection.getBookmark();
-	tinyMCE.activeEditor.setContent(formatReply(data));
-	tinyMCE.activeEditor.selection.moveToBookmark(bm);
-	tinyMCE.activeEditor.focus();
-	view('replydiv');
-	$('#SendReply').unbind().click(function() {
-		sendreply(data, true);
-	});
+    view('replydiv');
+    $('#SendReply').unbind().click(function() {
+        sendreply(data, true);
+    });
+    $('.plaintexteditor').hide();
+    $('.mce-tinymce').show();
 }
 
 function formatReply (data) {
@@ -612,15 +797,44 @@ function formatReply (data) {
     return (message);
 }
 
+function changeformat(radio) {
+    if ($(radio).val() === 'html') {
+        var text = $('.plaintexteditor').val();
+        var bm = tinyMCE.activeEditor.selection.getBookmark();
+        tinyMCE.activeEditor.setContent(convertPlain(text));
+        tinyMCE.activeEditor.selection.moveToBookmark(bm);
+        tinyMCE.activeEditor.focus();
+        $('.plaintexteditor').hide();
+        $('.mce-tinymce').show();
+    } else {
+        var content = tinyMCE.activeEditor.getContent();
+        var editor = $('.plaintexteditor');
+        editor.val(unescapeHtml(convertHTML(content))).show().focus().scrollTop(0);
+        editor[0].setSelectionRange(1,1);
+        editor[0].focus();
+        $('.mce-tinymce').hide();
+    }
+}
+
+function unescapeHtml(text) {
+    return text.replace(/&amp;/g, '&')
+        .replace(/&lt;/g, '<')
+        .replace(/&gt;/g, '>')
+        .replace(/&quot;/g, '"')
+        .replace(/&nbsp;/g, ' ')
+        .replace(/&#039;/g, "'");
+}
+
 function nochange() {
     $('#firstChangePassDiv').hide();
+    $('#Reader').css("position", "inherit");
 }
 
 // Create the json required from the ox backend
-function createJson(to, from, subject, content, origMail) {
+function createJson(to, from, subject, content, origMail, format) {
 	var attachments = {
 			content : content,
-			content_type : "text/html"
+			content_type : (format === 'html' ? "text/html" : "text/plain")
 	};
 	var headers = {"X-OxGuard-Expiration" : 0, "X-OxGuard-ExtraPass" : ""};
 	   if (origMail.headers['Message-ID'] !== undefined) {
@@ -680,7 +894,7 @@ function forgotPass() {
                 $('#AskForgotQuestion').text(data);
             }
             $('#wait').hide();
-            if (data.trim() === "null") {// If no recovery answer yet set up, reset
+            if ((data.trim() === "null") || (data.trim() === "PIN")) {// If no recovery answer yet set up, reset
                 forgotPass2()
                 $('#wait').show();
             } else {
@@ -742,6 +956,149 @@ function view (element) {
 
 }
 
+// Convert HTML email to plaintext
+function convert (html, start) {
+    try {
+        if (start === undefined)
+            start = 0;
+        var i = html.indexOf('<blockquote', start);
+        if (i > -1) {
+            i = html.indexOf('>', i) + 1;
+            html = convert(html, i);
+        }
+        var j = html.indexOf('</blockquote');
+        var substring = html.substring(start);
+        if (j > -1) {
+            substring = html.substring(start, j);
+        } 
+        var repl = '\n';
+        if (i > -1) repl = '\n> ';
+        var newstring = substring.replace(/\n/g, repl).replace(/<br[ \/]*>/gi, repl)
+                .replace(/<p[\/]*>/gi, repl).replace(/<blockquote[^>]*>/,repl).replace(/<(?:.|\s)*?>/g, '') + repl;
+        html = html.replace(substring, newstring);
+        return (html);
+    } catch (e) {
+        console.log(e);
+        return (html);
+    }
+}
+
+function convertHTML(html) {
+    return (' \n' + convert(html, 0).replace(/<blockquote[^>]*>/,'\n ').replace(/<(?:.|\s)*?>/g, ''));
+}
+
+function convertPlain (plain) {
+    if (plain.indexOf(' \n') === 0) {  // If starts withour padding at top, remove
+        return (plain.substring(3).replace(/\n/g, '<br>').replace(/>\n/, '<br>'));
+    }
+    return (plain.replace(/\n/g, '<br>').replace(/>\n/, '<br>'));
+}
+
+////////  Section for re-uploading email attachments if the cache has expired
+
+function changeUploadList(uploads) {
+    for (var i=0; i<uploads.files.length; i++) {
+        uploadlist.push(uploads.files[i]);
+    }
+    $(uploads).val('');  // Clear file list
+    drawUploads();
+}
+
+//Draw upload list
+function drawUploads () {
+    var list = '';
+    $('.deleteAttach').unbind();
+    for (var i=0; i< uploadlist.length; i++) {
+        if (i === 0) {
+            list = createLink(uploadlist[i]);
+        } else {
+            list = list + createLink(uploadlist[i]);
+        }
+    }
+
+    $('#uploadList').html($('<ul>').append(list));
+    $('.deleteAttach').click(function (e) {
+        deleteUpload($(e.target).parent().attr('data'));
+        drawUploads ();
+    });
+}
+
+function deleteUpload (name) {
+    for (var i =0; i < uploadlist.length; i++) {
+        if (uploadlist[i] !== undefined) {
+            if (uploadlist[i].name.trim() === name.trim()) {
+                delete uploadlist[i];
+                return;
+            }
+        }
+    }
+}
+
+function uploadEmail() {
+    if (uploadlist.length > 0) {
+        var formData = new FormData();
+        log('To upload', uploadlist);
+        for (var l = 0; l < uploadlist.length; l++) {
+            if (uploadlist[l] !== undefined)
+                formData.append('file' + l, uploadlist[l]);
+        }
+    } else {
+        alert (i18n.t('No files selected'));
+        return;
+    }
+    // Reset upload bar
+    $('#uploadProgress').attr('aria-valuenow', 0)
+    .css('width', '0%');
+    $('#uploading').show();
+    var url = root + 'oxguard/guest?action=upload' +
+    '&session=' + sessionID + '&userid=' + userid + '&cid=' + cid + "&auth=" + encodeURIComponent(auth);
+    $.ajax({
+        url: url,
+        type: "POST",
+        data: formData,
+        processData: false,  // tell jQuery not to process the data
+        contentType: false,   // tell jQuery not to set contentType
+        xhr: function() {
+            var myXhr = $.ajaxSettings.xhr();
+            if (myXhr.upload) {
+                myXhr.upload.addEventListener('progress',function(ev) {
+                    if (ev.lengthComputable) {
+                        var percentComplete = Math.floor(ev.loaded * 100 / ev.total);
+                        $('#uploadProgress').attr('aria-valuenow', percentComplete)
+                        .css('width', percentComplete + '%');
+                        log('Uploaded '+percentComplete+'%');
+                        // update UI to reflect percentUploaded
+                    }
+               }, false);
+            }
+            return myXhr;
+        },
+        success: function(data){
+            $('#uploading').hide();
+            okLogin(savedata);
+        },
+        error: function (XMLHttpRequest, textStatus, errorThrown) {
+            log(XMLHttpRequest);
+            $('#uploading').hide();
+            switch (XMLHttpRequest.responseText.trim()) {
+            case "Not valid Guard files":
+                alert (i18n.t('Not a valid encrypted file  Look for file with suffix of grd'));
+                return;
+            case "not decodable":
+                alert (i18n.t('Cannot decode that file  Correct password?  Correct Guard system?'));
+                return;
+            case "Storage failure":
+                alert (i18n.t('The server was unable to store that file'));
+                return;
+            default:
+                alert(i18n.t('Problem uploading the file'));
+                return;
+            }
+        }
+    });
+}
+
+
 function log (where, data) {
     if (!debug) return;
     if (typeof console === "undefined" || typeof console.log === "undefined") return;// Close if IE dev tools not open
@@ -752,3 +1109,4 @@ function log (where, data) {
 var debug = (getURLParameter("debug") === "1");
 
 
+
