@@ -76,7 +76,9 @@ import com.openexchange.guard.server.HandleArgs;
 public class Config {
 	private static Logger logger = LoggerFactory.getLogger(Config.class);
 
-	public static int version = 6;
+	public static int version = 7;  // Version for upgrader
+	
+	public static String serverVersion = "n/a";  // Value of server version from Java Manifest
 
     public static Level logLevel = Level.ALL;
     
@@ -185,6 +187,10 @@ public class Config {
     public static int max_idle_db = 2;
 
     public static int max_open_db = 10;
+    
+    public static int max_http_connections = 50;	// Max HTTP connections at once to a given route (backend, etc)
+    
+    public static int max_total_connections = 50;   // Max total HTTP pooled connections to all destinations
 
     public static boolean no_recovery = false;
     
@@ -197,6 +203,8 @@ public class Config {
     public static int bad_minute_lock = 10;
 
     public static int bad_password_count = 5;
+    
+    public static int max_remote = 100;  // Rate limit, max number of emails received remotely in lockout period
 
     public static int bad_ip_count = 10;
 
@@ -228,10 +236,16 @@ public class Config {
     
     public static String upsellTemplate = "/opt/open-xchange/guard/templates/upsell.html";
     
-    public static String passwordFromAddress = "";  // Address and name for the FROM header for new password emails
+    public static String passwordFromAddress = null;  // Address and name for the FROM header for new password emails
     
-    public static String passwordFromName = "";  
+    public static String passwordFromName = null;  
+    
+    public static boolean useStartTls = true;
+    
+    public static boolean showStatus = true;
 
+    public static String mailResolverUrl = null;  // MailResolverURL, must be defined so when email address is added to the end it will be valid
+    
     /**
      * Parse config file and update variables
      */
@@ -314,6 +328,13 @@ public class Config {
                             		deleteOnRetract = false;
                             	}
                             	break;
+                            case "com.openexchange.guard.showstatus":
+                            	if (data[1].trim().toLowerCase().contains("true")) {
+                            		showStatus = true;
+                            	} else {
+                            		showStatus = false;
+                            	}
+                            	break;
                             case "com.openexchange.guard.defaultoxguardenabled":
                                 if (data[1].trim().toLowerCase().contains("true")) {
                                     default_oxguard_enabled = true;
@@ -357,6 +378,13 @@ public class Config {
                             case "com.openexchange.guard.guestsmtpport":
                                 SMTPPort = Integer.parseInt(data[1].trim());
                                 break;
+                            case "com.openexchange.guard.usestarttls":
+                            	if (data[1].trim().toLowerCase().contains("true")) {
+                            		useStartTls = true;
+                            	} else {
+                            		useStartTls = false;
+                            	}
+                            	break;
                             case "com.openexchange.guard.jettyport":
                                 jettyport = Integer.parseInt(data[1].trim());
                                 break;
@@ -437,6 +465,15 @@ public class Config {
                             case "com.openexchange.guard.remotelogport":
                                 remotePort = Integer.parseInt(data[1].trim());
                                 break;
+                            case "com.openexchange.guard.mailresolverurl":
+                            	for (int i = 1; i < data.length; i++) {
+                            		if (i == 1) {
+                            			mailResolverUrl = data[i].trim();
+                            		} else {
+                            			mailResolverUrl = mailResolverUrl + "=" + data[i].trim();
+                            		}
+                            	}
+                            	break;
                             case "com.openexchange.guard.cronhour":
                                 cronHour = Integer.parseInt(data[1].trim());
                                 break;
@@ -452,6 +489,15 @@ public class Config {
                             case "com.openexchange.guard.badminutelock":
                                 bad_minute_lock = Integer.parseInt(data[1].trim());
                                 break;
+                            case "com.openexchange.guard.maxremote":
+                            	max_remote = Integer.parseInt(data[1].trim());
+                            	break;
+                            case "com.openexchange.guard.maxhttpconnections":
+                            	max_http_connections = Integer.parseInt(data[1].trim());
+                            	break;
+                            case "com.openexchange.guard.maxtotalconnections":
+                            	max_total_connections = Integer.parseInt(data[1].trim());
+                            	break;
                             case "com.openexchange.guard.shardsize":
                             	shard_size = Integer.parseInt(data[1].trim());
                             	break;
@@ -665,9 +711,9 @@ public class Config {
         return (OXBackend);
     }
     
-    public static String getBasicAuthenticationEncoding() {
+    public static String getBasicAuthenticationEncoding() throws UnsupportedEncodingException {
     	String userPassword = OXBackendUser + ":" + OXBackendPassword;
-        return new String(Base64.encodeBase64(userPassword.getBytes()));
+        return new String(Base64.encodeBase64(userPassword.getBytes("UTF-8")));
     }
 
     public static String getConfigDbConnectionString() {
