@@ -265,6 +265,20 @@ public class Access {
 
     }
     
+    public static void updatePin(int id, int cid, String pin) throws Exception {
+    	String command = "UPDATE og_KeyTable Set question = 'PIN', answer = ? WHERE id = ? AND cid = ?;";
+        try {
+            DbCommand com = new DbCommand(command);
+            com.addVariables(pin);
+            com.addVariables(id);
+            com.addVariables(cid);
+            DbQuery db = new DbQuery();
+            db.write(com, id, cid);
+            db.close();
+        } catch (SQLException e) {
+            logger.error("SQL error while updating keys", e);
+        }
+    }
 
     public GuardKeys getKeys(int id, int cid) throws Exception {
         int shard = 0;
@@ -307,6 +321,8 @@ public class Access {
                 key.settings = rs.getString("settings");
                 key.email = rs.getString("email");
                 key.lastup = rs.getDate("lastMod");
+                key.question = rs.getString("question");
+                key.answer = rs.getString("answer");
                 key.contextid = cid;
                 key.userid = id;
                 String recovery = rs.getString("Recovery");
@@ -454,7 +470,6 @@ public class Access {
                 output.append("{\"Email\" : \"" + email + "\", \"Count\" : \"" + getCountbyUser(ItemID, uid, ucid) + "\"}");
             }
             output.append("]");
-            System.out.println(output.toString());
         } catch (Exception ex) {
             logger.error("Error while getting count of encrypted items", ex);
         }
@@ -691,7 +706,8 @@ public class Access {
 
     public boolean storeQuestion(int id, int cid, String question, String answer) {
         EncrLib encr = new EncrLib();
-        String encranswer = encr.encryptAES(answer, Config.rpass, question, Config.AESKeyLen);
+        String encranswer = null;
+        if (answer != null) encranswer = encr.encryptAES(answer, Config.rpass, question, Config.AESKeyLen);
         String command = "UPDATE og_KeyTable SET question = ?, answer = ?" + " WHERE id = ? AND cid = ?";
         try {
             DbQuery db = new DbQuery();
