@@ -89,10 +89,13 @@ public class DbQuery {
     public OxResultSet rs = null;
 
     public boolean keepOpen = false;
+    
+    public int updated = -1;
 
     String transaction = null;
 
     int index = -1;
+    
 
     String query_id = "";
 
@@ -109,7 +112,7 @@ public class DbQuery {
     private JsonObject putRequest(String data, String database, int cid, boolean write) throws Exception {
 
         String url = "";
-        if (database.equals("oxdb") || database.equals(Config.configdbName)) {
+        if (database.equals("oxdb") || database.equals("configdb")) {
             url = "http" + (Config.backend_ssl ? "s" : "") + "://" + Config.getOxBackend() + ":" + Config.ox_backend_port + "/preliminary/database/v1/" + database + "/" + ((cid == 0) ? "" : cid + "/") + (write ? "writable" : "readOnly");
         } else {
             url = "http" + (Config.backend_ssl ? "s" : "") + "://" + Config.getOxBackend() + ":" + Config.ox_backend_port + "/preliminary/database/v1/pool/r/" + OGInit.read_map.get(database) + "/w/" + OGInit.write_map.get(database) + "/" + database + "/" + (write ? "writable" : "readOnly");
@@ -183,6 +186,8 @@ public class DbQuery {
             }
             if (hasError()) {
             	logger.error("Failed with url:" + url);
+            	logger.error("Status code:" + response.getStatusLine().getStatusCode());
+            	logger.error("Reason: " + response.getStatusLine().getReasonPhrase());
             	logger.error("Failed with query:" + data);
             	EntityUtils.consume(response.getEntity());
             	response.close();
@@ -195,6 +200,7 @@ public class DbQuery {
 
         }
         result = getJson(response);
+        checkUpdated();
         EntityUtils.consume(response.getEntity());
         response.close();
         putRequest.releaseConnection();
@@ -206,6 +212,18 @@ public class DbQuery {
         return (result);
     }
 
+    private void checkUpdated () {
+    	if (result == null) return;
+    	if (result.has("results")) {
+    		JsonObject sub = result.get("results").getAsJsonObject();
+    		if (sub.has(query_id)) {
+    			JsonObject qr = sub.get(query_id).getAsJsonObject();
+    			if (qr.has("updated")) {
+    				updated = qr.get("updated").getAsInt();
+    			}
+    		}
+    	}
+    }
     /**
      * Format the query into proper JSON for backend
      * 
@@ -283,7 +301,7 @@ public class DbQuery {
      */
     public JsonObject readFromConfigDb(DbCommand command) throws Exception {
         try {
-            return (putRequest(createQuery(command), Config.configdbName, 0, false));
+            return (putRequest(createQuery(command), "configdb", 0, false));
         } catch (Exception e) {
             throw (e);
         }
@@ -291,7 +309,7 @@ public class DbQuery {
 
     public JsonObject writeConfigDb(DbCommand command) throws Exception {
         try {
-            return (putRequest(createQuery(command), Config.configdbName, 0, true));
+            return (putRequest(createQuery(command), "configdb", 0, true));
         } catch (Exception e) {
             throw (e);
         }
@@ -531,5 +549,6 @@ public class DbQuery {
         rs = null;
         query_id = null;
         index = -1;
+        updated = -1;
     }
 }
