@@ -87,6 +87,9 @@ import com.openexchange.guard.cacheitems.Cache;
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.database.Access;
 import com.openexchange.guard.database.RecipKey;
+import com.openexchange.guard.exceptions.NoContentKeyException;
+import com.openexchange.guard.exceptions.NoItemException;
+import com.openexchange.guard.exceptions.NoMasterKeysException;
 import com.openexchange.guard.ox.InterOx;
 import com.openexchange.guard.server.CheckBad;
 import com.openexchange.guard.storage.Storage;
@@ -131,7 +134,7 @@ public class Crypto {
                     // user.cid = -user.cid;
                     logger.debug("Non OX user " + user.cid + " sending to filesystem");
  
-                    if (!Storage.saveEncrObj(user.userid, user.cid, itemID, addHeader(encrypted, itemID))) return (null);
+                    if (!Storage.saveEncrObj(user.userid, user.cid, itemID, addHeader(encrypted, itemID, data.length()))) return (null);
                 }
                 if (!saveContentKey(itemID, user.userid, user.cid, user.pubkey, contentKey, acc)) return (null);
                 acc.storeItemID(itemID, user.userid, user.cid, owner, owner_cid, exp, type, XML, salt);
@@ -227,12 +230,12 @@ public class Crypto {
                             user.userid,
                             user.cid,
                             itemID + "-" + filename.replace(".grd2", "").replace(".grd", ""),
-                            addHeader(encrypted, itemID));
+                            addHeader(encrypted, itemID, data.length()));
                     }
                 }
             }
             logger.info("Encrypted item " + itemID + " / " + start);
-            return (addHeader(encrypted, itemID));
+            return (addHeader(encrypted, itemID, data.length()));
         } catch (Exception e) {
             logger.error("Erroror while encrypting", e);
             return (null);
@@ -263,6 +266,7 @@ public class Crypto {
             }
             com.openexchange.guard.database.Access acc = new com.openexchange.guard.database.Access();
             com.openexchange.guard.database.RetrievedItem item = acc.getItemID(obj.ItemID, user, cid); // Get the item data
+            if (item == null) throw new NoItemException("Item not found");
             if (item.expiration > 0) {// If our expiration not set to unlimited, check
                 Date now = new Date();
                 if (item.expiration < now.getTime()) {
@@ -276,11 +280,12 @@ public class Crypto {
             EncrLib encryptor = new EncrLib();
             if ((contentkey == null) || (contentkey.equals("undefined"))) {
                 com.openexchange.guard.encr.GuardKeys keys = acc.getKeys(user, cid); // Load the users private key from store
+                if (keys == null) throw new NoMasterKeysException ("Unable to find/retrieve user keys");
                 contentkey = encryptor.decryptRSA(item.contentKey, keys.getDecodedPrivate(password));
                 if (contentkey == null) {
                     CheckBad.addBad(obj.ItemID);
                     // bad password
-                    return (null);
+                    throw new NoContentKeyException("Unable to decode content key");
                 }
                 logger.debug("Content key decoded");
             }
@@ -330,13 +335,14 @@ public class Crypto {
 
         return (null);
     }
+    
 
-    public byte[] addHeader(String strdata, String itemID) {
-        String filehead = StringUtils.rightPad("ID: " + itemID + ", Key: " + Config.AESKeyLen + ", START-----", 100, "-");// Create header
+    public static byte[] addHeader(String strdata, String itemID, int length) {
+        String filehead = StringUtils.rightPad("ID: " + itemID + ", Key: " + Config.AESKeyLen + (length > 0 ? (", Len: " + length) : "") + ", START-----", 100, "-");// Create header
         ByteArrayOutputStream out = new ByteArrayOutputStream();
         try {
-            out.write(filehead.getBytes());
-            out.write(strdata.getBytes());
+            out.write(filehead.getBytes("UTF-8"));
+            out.write(strdata.getBytes("UTF-8"));
         } catch (IOException e) {
             logger.error("IO error while adding header", e);
         }
