@@ -224,7 +224,7 @@ public class Api {
         	} catch (Exception e2) {
         		logger.error("unable to close failed connection at API getRequest", e2);
         	}
-            throw new RuntimeException("Failed : HTTP error code : " + response.getStatusLine().getStatusCode());
+            throw new RuntimeException("Failed : HTTP error code : " + response.getStatusLine().getStatusCode() + " " + response.getStatusLine().getReasonPhrase());
         }
         return (response);
     }
@@ -258,7 +258,7 @@ public class Api {
         	} catch (Exception e2) {
         		logger.error("unable to close failed connection at API putRequest", e2);
         	}
-            throw new RuntimeException("Failed : HTTP error code : " + response.getStatusLine().getStatusCode());
+            throw new RuntimeException("Failed : HTTP error code : " + response.getStatusLine().getStatusCode() + " " + response.getStatusLine().getReasonPhrase());
         }
         putRequest.releaseConnection();
         return (response);
@@ -292,7 +292,7 @@ public class Api {
         	} catch (Exception e2) {
         		logger.error("unable to close failed connection at API postRequest", e2);
         	}
-            throw new RuntimeException("Failed : HTTP error code : " + response.getStatusLine().getStatusCode());
+            throw new RuntimeException("Failed : HTTP error code : " + response.getStatusLine().getStatusCode() + " " + response.getStatusLine().getReasonPhrase());
         }
         BufferedReader reader = new BufferedReader(new InputStreamReader(response.getEntity().getContent(), "UTF-8"));
         String json = reader.readLine();
@@ -446,6 +446,19 @@ public class Api {
 	        }
         return (null);
     }
+    
+    public JsonObject saveDraftMail(JsonObject mail, ArrayList<Attachment> attachments) {
+        String params = "mail?action=new&session=" + session;
+        Gson gson = new Gson();
+        try {
+            return (multipartPost(gson.toJson(mail.getAsJsonObject("data")), params, attachments));
+        } catch (ClientProtocolException e) {
+        	logger.error("Error saving sent mail", e);
+        } catch (IOException e) {
+        	logger.error("IO error saving sent mail", e);
+        }
+    return (null);
+}
 
     public JsonObject saveInBox(JsonObject mail, ArrayList<Attachment> attachments) {
         String params = "mail?action=new";
@@ -484,7 +497,41 @@ public class Api {
         }
         return (null);
     }
+    
+    public Attachment getPlainAttachment (int id, String attachment, String folder) {
+    	long start = System.currentTimeMillis();
+        String params = "mail?action=attachment" + "&folder=" + URLEncoder.encode(folder) + "&id=" + id + "&attachment=" + attachment;
+        try {
+            CloseableHttpResponse response = getResponse(params);
+            HttpEntity ent = response.getEntity();
+            Attachment att = new Attachment();
+            att.content = EntityUtils.toByteArray(ent);
+            logger.info("Retrieved ox item at " + start);
+            return (att);
 
+        } catch (IllegalStateException e) {
+        	logger.error("Error getting attachment", e);
+        } catch (IOException e) {
+        	logger.error("IO error getting attachment", e);
+        }
+        return (null);
+    }
+
+    public String getMime (String ref) {
+    	int i = ref.lastIndexOf(("/"));
+    	if (i < -1) return (null);
+    	String folder = ref.substring(0, i);
+    	String id = ref.substring(i+1);
+    	int emailid;
+    	try {
+    		emailid = Integer.parseInt(id);
+    	} catch (Exception ex) {
+    		logger.error("Error parsing email id from msgref " + ref);
+    		return (null);
+    	}
+    	return getMime(emailid, folder);
+    }
+    
     public String getMime(int id, String folder) {
         String params = "mail?action=get" + "&folder=" + URLEncoder.encode(folder) + "&id=" + id + "&src=1&save=1";
         try {
@@ -721,8 +768,11 @@ public class Api {
     public Attachment getInlineImage (String url) {
     	try {
     		url = java.net.URLDecoder.decode(url, "UTF-8").replace("&amp;","&");
-    		if (url.contains("file?action")) {
-	        	String params = url.substring(url.indexOf("file?action"));
+    		if (url.contains("file?action") || url.contains("image/mail")) {
+    			int index = 0;
+    			if (url.contains("file?action")) index = url.indexOf("file?action");
+    			if (url.contains("image/mail")) index = url.indexOf("image/mail");
+	        	String params = url.substring(index);
 	            Attachment attach = new Attachment();
 	            CloseableHttpResponse response = getResponse(params);
 	            HttpEntity ent = response.getEntity();
@@ -778,7 +828,7 @@ public class Api {
     }
     
     public void deleteItem(int id, int folder) {
-        String params = "files?action=delete&timestamp=2116800000000";
+        String params = "files?action=delete&hardDelete=true&timestamp=2116800000000";
         try {
             String data = "[{\"id\":\"" + id + "\",\"folder\":\"" + folder + "\"}]";
             CloseableHttpResponse response = putRequest(params, data);
@@ -800,6 +850,7 @@ public class Api {
 	    		if (!msgref.startsWith(draft.get("data").getAsString())) return;  // If not from default draft folder, then exit (reply or forward)
 	    		String id = msgref.substring(i + 1);
 	    		String folder = msgref.substring(0, i);
+	    		
 	    		String params = "mail?action=delete&timestamp=2116800000000";
 	    		JsonObject data = new JsonObject();
 	    		data.addProperty("folder", folder);
