@@ -53,8 +53,11 @@ import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStreamReader;
+import java.io.UnsupportedEncodingException;
 import java.net.URLEncoder;
+import java.util.Calendar;
 import java.util.Date;
+import java.util.GregorianCalendar;
 
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
@@ -86,6 +89,8 @@ import com.openexchange.guard.database.RecipKey;
 import com.openexchange.guard.dbpool.ContextDBConnection;
 import com.openexchange.guard.dbpool.Manager;
 import com.openexchange.guard.encr.GuardKeys;
+import com.openexchange.guard.exceptions.GuardMissingParameter;
+import com.openexchange.guard.util.Core;
 
 
 /*
@@ -100,7 +105,9 @@ public class InterOx {
                 email = email.substring(email.indexOf("@") + 1);
             InterOxInfo info = incache(email);
             if (info != null) {
-                return (info);
+            	if (compareSqlDate(info.lastcheck)) {  // If checked today, then use, else reload
+        			return (info);
+        		}
             }
             String url = null;
             Record[] records = new Lookup("_oxguard." + email, Type.TXT).run();
@@ -148,11 +155,12 @@ public class InterOx {
     }
 
     private InterOxInfo saveInCache(String domain, String url) {
-        String command = "INSERT INTO iox (domain, url, lastcheck) VALUES (?, ?, NOW())";
+        String command = "INSERT INTO iox (domain, url, lastcheck) VALUES (?, ?, NOW()) ON DUPLICATE KEY UPDATE url = ?, lastcheck = NOW()";
         try {
             DbCommand com = new DbCommand(command);
             com.addVariables(domain);
             com.addVariables(url);
+            com.addVariables(url);
             DbQuery db = new DbQuery();
             db.writeOxGuard(com);
             db.close();
@@ -166,24 +174,24 @@ public class InterOx {
 
     // ////// Receiving end of interox
 
-    public void getKey(HttpServletRequest request, HttpServletResponse response) {
-        String from = request.getParameter("from");
-        String token = request.getParameter("token");
-        String email = request.getParameter("email");
-        if (com.openexchange.guard.server.CheckBad.isBad(request.getRemoteAddr(), Config.bad_ip_count)) {
+    /**
+     * Get public Key of email address.  Will not create key unless from and token populated
+     * @param request
+     * @param response
+     * @throws GuardMissingParameter
+     */
+    public void getKey(HttpServletRequest request, HttpServletResponse response) throws GuardMissingParameter {
+        String from = Core.getStringParameter(request, "from");
+        String token = Core.getStringParameter(request, "token");
+        String email = Core.getStringParameter(request, "email", true);
+        if (com.openexchange.guard.server.CheckBad.isBad(Core.getIP(request), Config.bad_ip_count)) {
             sendFail(response, "Temporary lockout");
             return;
         }
-        if (email == null) {
-            sendFail(response, "Missing Email");
-            return;
-        }
-        ;
         if (!email.contains("@")) {
             sendFail(response, "Bad email format");
             return;
         }
-        ;
         try {
             boolean create = false;
             if ((from != null) && (token != null)) {
@@ -196,7 +204,7 @@ public class InterOx {
                             create = true; // Token verifies with what we have in storage, ok to create
                         } else {
                         	logger.error("Token mismatch.  Requesting server shows token " + token + ", expecting " + info.token);
-                        	if (info.lastcheck.compareTo(new Date()) < 0) {
+                        	if (!compareSqlDate(info.lastcheck)) {
                         		logger.info("Token not verified today, trying again");
                         		create = verifyToken(info, token);
                         		if (create) logger.info("OK Verified");
@@ -208,7 +216,7 @@ public class InterOx {
             RecipKey keys = null;
             if (create) {
                 GetPubKey gk = new GetPubKey();
-                keys = gk.getPubKey(email, 0, from, true, true);
+                keys = gk.getPubKey(email, 0, 0, from, true, true);
             } else {
                 keys = simipleCheck(email);
             }
@@ -220,7 +228,7 @@ public class InterOx {
                 	sendOK(response, (Base64.encodeBase64String(keys.pubkey.getEncoded())));
                 }
             } else {
-                com.openexchange.guard.server.CheckBad.addBad(request.getRemoteAddr());
+                com.openexchange.guard.server.CheckBad.addBad(Core.getIP(request));
                 sendFail(response, "Unable to create keys");
             }
         } catch (Exception ex) {
@@ -234,6 +242,10 @@ public class InterOx {
         logger.debug("Receiving item");
         String xml = getRequest(request);
         logger.debug("XML: " + xml);
+        if (com.openexchange.guard.server.CheckBad.isBad("r-" + Core.getIP(request), Config.bad_ip_count)) {
+            sendFail(response, "Temporary lockout");
+            return;
+        }
         DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
         try {
             DocumentBuilder builder = factory.newDocumentBuilder();
@@ -251,9 +263,13 @@ public class InterOx {
                 sendFail(response, "Bad data");
                 return;
             }
+            if (com.openexchange.guard.server.CheckBad.isBad("r-" + email, Config.max_remote)) {  // This is checking for rate limit
+                sendFail(response, "Temporary lockout");
+                return;
+            }
             logger.debug("Storing");
             GetPubKey gk = new GetPubKey();
-            RecipKey keys = gk.getPubKey(email, 0, null, false, true);
+            RecipKey keys = gk.getPubKey(email, 0, 0, null, false, true);
             if (keys != null) {
                 if (keys.pubkey == null) {
                     sendFail(response, "Unable to create keys");
@@ -263,43 +279,46 @@ public class InterOx {
                     acc.storeContentKey(itemid, keys.userid, keys.cid, contentKey);
                     logger.info("Recieved item " + itemid);
                     sendOK(response, "OK");
+                    com.openexchange.guard.server.CheckBad.addBad("r-" + email);  // Here we are adding email to bad, not because fail, but for rate limit
                 }
             } else {
+            	com.openexchange.guard.server.CheckBad.addBad("r-" + Core.getIP(request));
                 sendFail(response, "Fail");
             }
 
         } catch (Exception ex) {
             logger.error("Error saving item", ex);
+            com.openexchange.guard.server.CheckBad.addBad("r-" + Core.getIP(request));
             sendFail(response, "Bad request");
         }
 
     }
 
-    public void verifyToken(HttpServletRequest request, HttpServletResponse response) {
-        String token = request.getParameter("token");
-        if (!com.openexchange.guard.server.CheckBad.isBad(request.getRemoteAddr(), Config.bad_ip_count)) {
+    public void verifyToken(HttpServletRequest request, HttpServletResponse response) throws GuardMissingParameter {
+        String token = Core.getStringParameter(request, "token", true);
+        if (!com.openexchange.guard.server.CheckBad.isBad(Core.getIP(request), Config.bad_ip_count)) {
             if (token.trim().equals(Config.token.trim())) {
                 com.openexchange.guard.util.Core.sendOK(response, "true");
                 return;
             }
         }
-        com.openexchange.guard.server.CheckBad.addBad(request.getRemoteAddr());
+        com.openexchange.guard.server.CheckBad.addBad(Core.getIP(request));
         com.openexchange.guard.util.Core.sendOK(response, "false");
     }
 
-    public void getReadCount(HttpServletRequest request, HttpServletResponse response) {
-        String email = request.getParameter("email");
-        String itemid = request.getParameter("item");
-        if ((email == null) || (itemid == null)) {
-            sendFail(response, "Missing Email/item");
-            return;
-        }
-        ;
+    public void getReadCount(HttpServletRequest request, HttpServletResponse response) throws GuardMissingParameter {
+        String email = Core.getStringParameter(request, "email", true);
+        String itemid = Core.getStringParameter(request, "item", true);
+
         if (!email.contains("@")) {
             sendFail(response, "Bad email format");
+            com.openexchange.guard.server.CheckBad.addBad(Core.getIP(request));
             return;
         }
-        ;
+        if (com.openexchange.guard.server.CheckBad.isBad(Core.getIP(request), Config.bad_ip_count)) {
+        	sendFail(response, "Lockout");
+        	return;
+        }
         com.openexchange.guard.database.Access acc = new com.openexchange.guard.database.Access();
         try {
             GuardKeys keys = acc.getKeysFromEmail(email);
@@ -314,10 +333,15 @@ public class InterOx {
                 db.read(com, keys.userid, keys.contextid);
                 if (db.next()) {
                     int data = db.rs.getInt("Status");
+                    if (Config.showStatus == false) {
+                    	if (data > -1) data = -9999;
+                    }
                     sendOK(response, Integer.toString(data));
 
                 }
                 db.close();
+            } else {
+            	com.openexchange.guard.server.CheckBad.addBad(Core.getIP(request));
             }
         } catch (Exception ex) {
             logger.error("Error getting read count for item " + itemid + " for user " + email, ex);
@@ -326,25 +350,25 @@ public class InterOx {
 
     }
 
-    public boolean deleteshare(HttpServletRequest request, HttpServletResponse response) {
-    	String email = request.getParameter("email");
-    	String itemid = request.getParameter("item");
-    	String salt = request.getParameter("salt");
+    public boolean deleteshare(HttpServletRequest request, HttpServletResponse response) throws GuardMissingParameter {
+    	String email = Core.getStringParameter(request, "email", true);
+    	String itemid = Core.getStringParameter(request, "item", true);
+    	String salt = Core.getStringParameter(request, "salt", true);
         try {
-            if ((email == null) || (itemid == null)) {
-                sendFail(response, "Missing Email/item");
-                return (false);
-            }
-            ;
+
             if (!email.contains("@")) {
                 sendFail(response, "Bad email format");
+                com.openexchange.guard.server.CheckBad.addBad(Core.getIP(request));
                 return (false);
             }
-            ;
+            if (com.openexchange.guard.server.CheckBad.isBad(Core.getIP(request), Config.bad_ip_count)) {
+            	sendFail(response, "Lockout");
+            	return (false);
+            }
             com.openexchange.guard.database.Access acc = new com.openexchange.guard.database.Access();
             GuardKeys keys = acc.getKeysFromEmail(email);
             if (keys != null) {
-                String command = "DELETE og.* FROM og_content_keys og INNER JOIN og_encrypted_items encr ON og.ItemId = encr.ID WHERE ItemId = '" + "? AND UserId = ? AND CID = ?" + " AND encr.Salt = ?";
+                String command = "DELETE og.* FROM og_content_keys og INNER JOIN og_encrypted_items encr ON og.ItemId = encr.ID WHERE ItemId = ? AND UserId = ? AND CID = ? AND encr.Salt = ?";
                 DbCommand com = new DbCommand(command);
                 com.addVariables(itemid);
                 com.addVariables(keys.userid);
@@ -352,21 +376,26 @@ public class InterOx {
                 com.addVariables(salt);
                 DbQuery db = new DbQuery();
                 db.write(com, keys.userid, keys.contextid);
-                logger.debug(command);
+                logger.debug("updated: " + db.updated);
+                String resp = "OK";
+                if (db.updated < 1) resp = "No";
                 db.close();
+                Core.sendOK(response, resp);
                 return (true);
             }
         } catch (Exception ex) {
             logger.error("Error deleting share of item " + itemid + " for user " + email, ex);
 
         }
+        com.openexchange.guard.server.CheckBad.addBad(Core.getIP(request));
+        Core.sendFail(response, "Fail");
         return (false);
     }
 
-    public boolean retract(HttpServletRequest request, HttpServletResponse response) {
-    	String email = request.getParameter("email");
-    	String itemid = request.getParameter("item");
-    	String salt = request.getParameter("salt");
+    public boolean retract(HttpServletRequest request, HttpServletResponse response) throws GuardMissingParameter {
+    	String email = Core.getStringParameter(request, "email", true);
+    	String itemid = Core.getStringParameter(request, "item", true);
+    	String salt = Core.getStringParameter(request, "salt", true);
     	boolean delete = Config.deleteOnRetract;
     	try {
     		if (request.getParameter("delete") != null) {
@@ -376,16 +405,11 @@ public class InterOx {
     		logger.error("Error parsing retract request", e);
     	}
         try {
-            if ((email == null) || (itemid == null)) {
-                sendFail(response, "Missing Email/item");
-                return (false);
-            }
-            ;
             if (!email.contains("@")) {
                 sendFail(response, "Bad email format");
+                com.openexchange.guard.server.CheckBad.addBad(Core.getIP(request));
                 return (false);
             }
-            ;
             com.openexchange.guard.database.Access acc = new com.openexchange.guard.database.Access();
             GuardKeys keys = acc.getKeysFromEmail(email);
             if (keys != null) {
@@ -400,6 +424,7 @@ public class InterOx {
             logger.error("Error retracting item " + itemid + " for user " + email, ex);
 
         }
+        com.openexchange.guard.server.CheckBad.addBad(Core.getIP(request));
         return (false);
 
     }
@@ -516,7 +541,8 @@ public class InterOx {
             BufferedReader reader = new BufferedReader(new InputStreamReader(response.getEntity().getContent(), "UTF-8"));
             String answer = reader.readLine();
             reader.close();
-            return (true);
+            if (answer.contains("OK")) return (true);
+            return(false);
         } catch (Exception ex) {
             logger.error("Error while deleting share of item " + item + " for user " + email, ex);
             return (false);
@@ -582,7 +608,7 @@ public class InterOx {
 
     }
 
-    public boolean sendItem(String email, String itemID, String contentKey, long exp, int type, String XML, String salt) {
+    public boolean sendItem(String email, String itemID, String contentKey, long exp, int type, String XML, String salt) throws UnsupportedEncodingException {
         StringBuilder data = new StringBuilder();
         data.append("<item>");
         data.append("<email>" + StringEscapeUtils.escapeXml11(email) + "</email>");
@@ -590,7 +616,7 @@ public class InterOx {
         data.append("<contentkey>" + contentKey + "</contentkey>");
         data.append("<expiration>" + exp + "</expiration>");
         data.append("<type>" + type + "</type>");
-        data.append("<xml>" + Base64.encodeBase64String(XML.getBytes()) + "</xml>");
+        data.append("<xml>" + Base64.encodeBase64String(XML.getBytes("UTF-8")) + "</xml>");
         data.append("<salt>" + salt + "</salt>");
         data.append("</item>");
         String url = getOXLocation(email).url + "interox?action=saveitem";
@@ -645,5 +671,20 @@ public class InterOx {
         }
         return (null);
     }
+    
+    /**
+     * Check sql date with just date, not time, to current date
+     * @param time
+     * @return
+     */
+    private boolean compareSqlDate (Date time) {
+    	Calendar now = new GregorianCalendar();
+    	now.set(Calendar.HOUR, 0);
+    	now.set(Calendar.MINUTE, 0);
+    	now.set(Calendar.SECOND, 0);
+    	now.set(Calendar.MILLISECOND, 0);
+    	return (time.compareTo(now.getTime()) == 0);
+
+    }
 
 }
