@@ -52,6 +52,7 @@ package com.openexchange.guard.server;
 import java.io.BufferedReader;
 import java.io.IOException;
 import java.net.URLEncoder;
+import java.security.PrivateKey;
 
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletRequest;
@@ -70,51 +71,30 @@ import com.openexchange.guard.config.OxConfiguration;
 import com.openexchange.guard.database.Access;
 import com.openexchange.guard.database.DbCommand;
 import com.openexchange.guard.database.DbQuery;
+import com.openexchange.guard.database.GetPubKey;
 import com.openexchange.guard.database.Keymanager;
+import com.openexchange.guard.database.RecipKey;
 import com.openexchange.guard.database.Sharding;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.GuardKeys;
 import com.openexchange.guard.encr.PgpKeys;
+import com.openexchange.guard.exceptions.BadPasswordException;
+import com.openexchange.guard.exceptions.GuardMissingParameter;
+import com.openexchange.guard.mailcreator.Mail;
 import com.openexchange.guard.ox.Api;
 import com.openexchange.guard.ox.Languages;
+import com.openexchange.guard.ox.Upgrader;
 import com.openexchange.guard.util.Core;
 import com.openexchange.guard.validator.EmailValidator;
 
-final class Settings {
-	public enum permissions {
-        NONE, MAIL, FILE, BOTH
-    };
-    
-    public permissions perm = permissions.NONE;
-    public boolean extrapass = Config.ok_extra_password;  // Set to default config level
-    public boolean privatepass = true;
-    public boolean deleteRecovery = true;
-    public boolean deleteKeyOnRevoke = Config.deleteOnRetract;
-    
-}
 
 public class Auth {
 	
 	private static Logger logger = LoggerFactory.getLogger(Auth.class);
 
-    public JsonObject getJSON(HttpServletRequest request) {
-        StringBuffer sb = new StringBuffer();
-        String line = null;
-        try {
-            BufferedReader reader = request.getReader();
-            while ((line = reader.readLine()) != null) {
-                sb.append(line);
-            }
-        } catch (Exception e) {
-            logger.error("Error using JSON reader", e);
-        }
-        JsonParser jsonParser = new JsonParser();
-        JsonObject json = (JsonObject) jsonParser.parse(sb.toString());
-        return (json);
-    }
 
     public void createKeys(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws Exception {
-        JsonObject json = getJSON(request);
+        JsonObject json = Core.getJSON(request);
     	Api ap = new Api(cookie, request);
         if (ap.verifyLogin() == false) {
         	logger.error("Create keys but not logged into UI");
@@ -123,13 +103,20 @@ public class Auth {
         }
         logger.debug("Starting to create master key");
         Access acc = new Access();
-        int userid = json.get("user_id").getAsInt();
-        int cid = json.get("cid").getAsInt();
-        String language = json.get("language").getAsString();
-        String password = json.get("password").getAsString();
+        int userid = Core.getIntFromJson(json, "user_id", true);
+        int cid = Core.getIntFromJson(json, "cid", true);
+        String language = Core.getStringFromJson(json, "language");
+        String password = Core.getStringFromJson(json, "password", true);
         GuardKeys keys = acc.getKeys(userid, cid);
         if (keys == null) { // make sure already don'e exist
             String email = acc.getEmail(userid, cid);
+            // Check we don't already have keys for that email
+            GuardKeys prevKey = acc.getKeysFromEmail(email);
+            if (prevKey != null) {
+            	logger.error("Attempting to create keys that exist in other context");
+            	Core.sendFail(response, "Existing");
+            	return;
+            }
             String name = email;
             if (json.get("name") != null) {
                 name = json.get("name").getAsString();
@@ -141,7 +128,10 @@ public class Auth {
                 response.getWriter().write(" {\"result\":\"fail \"} ");
             } else {
             	if (json.has("email")) {
-            		acc.storeQuestion(userid, cid, "e", json.get("email").getAsString());
+            		String recEmail = json.get("email").getAsString();
+            		if (EmailValidator.validate(recEmail)) {
+            			acc.storeQuestion(userid, cid, "e", recEmail);
+            		}
             	}
                 logger.debug("Success creating master key");
                 response.getWriter().write(" {\"result\":\"ok\"} ");
@@ -158,18 +148,18 @@ public class Auth {
      * @throws Exception
      */
     public String changepass(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws Exception {
-        JsonObject json = getJSON(request);
+        JsonObject json = Core.getJSON(request);
         if (CheckBad.isBad(cookie.JSESSIONID, Config.bad_password_count)) {
         	logger.debug("Lockout for bad attempts");
             response.getWriter().write("{\"auth\":\"Lockout\"}");
             return (null);
         }
-        String newpass = json.get("newpass").getAsString().trim();
+        String newpass = Core.getStringFromJson(json, "newpass", true).trim();
         if (newpass.length() < Config.min_password_length) {
         	response.getWriter().write("{\"auth\":\"Bad new password\",\"minlength\":" + Config.min_password_length + "}");
             return (null);
         }
-        int cid = json.get("cid").getAsInt();
+        int cid = Core.getIntFromJson(json, "cid", true);
         String question = "";
         if (json.has("question")) { // Sent with new question and answer?
             question = json.get("question").getAsString();
@@ -195,7 +185,7 @@ public class Auth {
         response.setContentType("application/json");
         response.addHeader("Access-Control-Allow-Origin", "*");
         logger.debug("change password request");
-        int userid = json.get("user_id").getAsInt();
+        int userid = Core.getIntFromJson(json, "user_id", true);
 
         com.openexchange.guard.database.Access acc = new com.openexchange.guard.database.Access();
         GuardKeys keys = acc.getKeys(userid, cid);
@@ -204,7 +194,7 @@ public class Auth {
         	return (null);
         }
         String oldpass = "";
-        oldpass = json.get("oldpass").getAsString().trim();
+        oldpass = Core.getStringFromJson(json, "oldpass", true).trim();
         PgpKeys keygen = new PgpKeys();
         keys = keygen.changePassword(oldpass, newpass, keys);
         if (keys == null) {
@@ -243,9 +233,9 @@ public class Auth {
      * @throws Exception
      */
     public void changeSecondaryEmail (HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws Exception {
-    	JsonObject json = getJSON(request);
-    	int cid = json.get("cid").getAsInt();
-    	int userid = json.get("userid").getAsInt();
+    	JsonObject json = Core.getJSON(request);
+    	int cid = Core.getIntFromJson(json, "cid", true);
+    	int userid = Core.getIntFromJson(json, "userid", true);
     	if (CheckBad.isBad(Integer.toString(userid) + "-" + Integer.toString(cid), Config.bad_password_count)) {
     		Core.sendFail(response, "Lockout");
     		return;
@@ -255,8 +245,8 @@ public class Auth {
             Core.sendFail(response, "Not logged in");
             return;
         }
-        String password = json.get("password").getAsString();
-        String newemail = json.get("email").getAsString();
+        String password = Core.getStringFromJson(json, "password", true);
+        String newemail = Core.getStringFromJson(json, "email", true);
         Access acc = new Access();
         GuardKeys keys = acc.getKeys(userid, cid);
         if (keys.getDecodedPrivate(password) != null) {
@@ -277,8 +267,8 @@ public class Auth {
             Core.sendFail(response, "Not logged in");
             return;
         }
-        int cid = Integer.parseInt(request.getParameter("cid"));
-        int userid = Integer.parseInt(request.getParameter("userid"));
+        int cid = Core.getIntParameter(request, "cid", true);
+        int userid = Core.getIntParameter(request, "userid", true);
        	String command = "SELECT answer, email FROM og_KeyTable WHERE id = ? AND CID = ?";
     	DbCommand dcom = new DbCommand(command);
     	dcom.addVariables(userid);
@@ -305,8 +295,8 @@ public class Auth {
     }
 
     public void resetPass(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws Exception {
-    	int cid = Integer.parseInt(request.getParameter("cid"));
-        int userid = Integer.parseInt(request.getParameter("userid"));
+    	int cid = Core.getIntParameter(request, "cid", true);
+        int userid = Core.getIntParameter(request, "userid", true);
         resetPass (userid, cid, request, response, cookie, true);
     }
     
@@ -330,7 +320,7 @@ public class Auth {
 					}
 	    			Sender sender = new Sender();
 	                com.openexchange.guard.mailcreator.Mail mf = new com.openexchange.guard.mailcreator.Mail();
-	                if (!sender.send(mf.getResetEmail(email, email, newpass, Config.default_lang, 0), null)) {
+	                if (!sender.send(mf.getResetEmail(email, Mail.getFromAddress(email, email, keys.userid, keys.contextid), newpass, Config.default_lang, 0), null)) {
 	                	System.out.println("Failed to send");
 	                } else {
 	                	System.out.println("Reset sent");
@@ -343,6 +333,27 @@ public class Auth {
     	
     }
     
+    // Remove PIN, for command line use only
+    public static void removePin (String email) {
+    	Access acc = new Access();
+		try {
+			GuardKeys keys = acc.getKeysFromEmail(email);
+			if (keys == null) {
+				System.out.println("NO ACCOUNT FOUND");
+				return;
+			}
+			if (keys.contextid > 0) { // OX email
+				System.out.println("Not a guest account");
+				return;
+			}
+		    acc.storeQuestion(keys.userid, keys.contextid, null, null);
+		    System.out.println("done");
+		} catch (Exception ex) {
+			System.out.println("Error resetting PIN");
+			System.out.println(ex);
+		}
+    }
+    
     
     public void resetPass(int userid, int cid, HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie, boolean web) throws Exception {
     	int templId = 0;
@@ -392,7 +403,8 @@ public class Auth {
     			}
     			Sender sender = new Sender();
                 com.openexchange.guard.mailcreator.Mail mf = new com.openexchange.guard.mailcreator.Mail();
-                if (!sender.send(mf.getResetEmail(secondEmail, (defaultEmail == null ? secondEmail : defaultEmail), newpass, lang, templId), null)) {
+                String from = (defaultEmail == null ? secondEmail : defaultEmail);
+                if (!sender.send(mf.getResetEmail(secondEmail, Mail.getFromAddress(from, from, userid, cid) , newpass, lang, templId), null)) {
                     if (web) {
                     	Core.sendOK(response, "FailNotify");
                     } else System.out.println("Failed to send nofication");
@@ -427,11 +439,11 @@ public class Auth {
      * @throws Exception
      */
     public String changepassFromOxRecovery(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws Exception {
-        JsonObject json = getJSON(request);
-        int cid = json.get("cid").getAsInt();
-        String password = json.get("oxpassword").getAsString();
-        int userid = json.get("user_id").getAsInt();
-        String user = json.get("user").getAsString();
+        JsonObject json = Core.getJSON(request);
+        int cid = Core.getIntFromJson(json, "cid", true);
+        String password = Core.getStringFromJson(json, "oxpassword", true);
+        int userid = Core.getIntFromJson(json, "user_id", true);
+        String user = Core.getStringFromJson(json, "user", true);
         com.openexchange.guard.database.Access acc = new com.openexchange.guard.database.Access();
         GuardKeys keys = acc.getKeys(userid, cid);
         // if ox member, verify actually logged in to ui
@@ -451,7 +463,7 @@ public class Auth {
         // OK, logged in , so we can change
 
         Keymanager keyman = new Keymanager();
-        String newpass = json.get("newpass").getAsString();
+        String newpass = Core.getStringFromJson(json, "newpass", true);
         String resetpassword = keyman.resetPassword(keys.email, newpass);
         response.setContentType("application/json");
         response.addHeader("Access-Control-Allow-Origin", "*");
@@ -478,9 +490,10 @@ public class Auth {
      * 
      * @param request
      * @param response
+     * @throws GuardMissingParameter 
      */
-    public void guestlogin(HttpServletRequest request, HttpServletResponse response) {
-    	JsonObject login_json = getJSON(request);
+    public void guestlogin(HttpServletRequest request, HttpServletResponse response) throws GuardMissingParameter {
+    	JsonObject login_json = Core.getJSON(request);
         com.openexchange.guard.database.Access acc = new com.openexchange.guard.database.Access();
         response.setContentType("application/json");
         response.addHeader("Access-Control-Allow-Origin", "*");
@@ -491,14 +504,15 @@ public class Auth {
         Cookie cookie = new Cookie("OxReaderID", cookieID);
         cookie.setHttpOnly(true);
         response.addCookie(cookie);
-        String username = login_json.get("username").getAsString();
+        String username = Core.getStringFromJson(login_json, "username", true);
+        String pin = login_json.get("pin") == null ? null : login_json.get("pin").getAsString();
         try {
             if (CheckBad.isBad(username, Config.bad_password_count)) {
             	response.getWriter().write("{\"auth\":\"lockout\", \"time:\": " + Config.bad_minute_lock + "}");
             	logger.debug("Guest auth lockout for " + username);
                 return;
             }
-            String password = login_json.get("password").getAsString();
+            String password = Core.getStringFromJson(login_json, "password", true);
             GuardKeys keys = acc.getKeysFromEmail(username);
             if (keys == null) {
                 response.getWriter().write("{\"auth\":\"No Key\"}");
@@ -521,15 +535,39 @@ public class Auth {
             String json = "{\"user_id\" : \"" + keys.userid + "\", \"encr_password\" : \"" + password + "\", \"sessionID\" : \"" + sessionID + "\", \"cid\" : \"" + keys.contextid + "\" , \"lastMod\" : \"" + keys.lastup + "\"}";
             EncrLib encr = new EncrLib();
             String returndata = "Bad Password";
-            if (keys.getDecodedPrivate(password) != null) {
+            PrivateKey key = null;
+            
+            try {
+            	key = keys.getDecodedPrivate(password);
+            } catch (BadPasswordException ex) {
+            	logger.debug("Bad password for " + username);
+            }
+            if (key != null) {
                 // returndata = encr.encryptAES(json, token, cookieID, Config.AESKeyLen);
-                returndata = encr.encryptAES(json, token);
+                
                 logger.debug("Guest auth good for " + username);
+                // Check if PIN
+                
+                if ((keys.lastup == null) && (keys.question == null ? false : keys.question.equals("PIN"))) {
+                	if (pin != null) {
+                		if (pin.equals(keys.answer)) {
+                			returndata = encr.encryptAES(json, token);
+                		} else {
+                			returndata = "Bad";
+                			CheckBad.addBad(username);
+                		}
+                	} else {
+                		logger.debug("New Guest login, pin required");
+                		returndata = "PIN";
+                	}
+                } else {
+                	returndata = encr.encryptAES(json, token);
+                }
             } else {
             	CheckBad.addBad(username);
             	logger.debug("Guest auth bad for " + username);
             }
-
+            
             response.getWriter().write(
                 " {\"auth\":\"" + returndata + "\", \"userid\":\"" + keys.userid + "\",\"cid\":\"" + keys.contextid + "\",\"sessionID\" : \"" + sessionID + "\", \"lastMod\" : \"" + keys.lastup + "\"}");
 
@@ -547,7 +585,7 @@ public class Auth {
     public void login(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws IOException {
         try {
 
-            JsonObject json = getJSON(request);
+            JsonObject json = Core.getJSON(request);
             Settings settings = getPermissions(cookie, request);
             login(json, request, response, cookie, settings);
         } catch (Exception ex) {
@@ -557,7 +595,7 @@ public class Auth {
         }
     }
 
-    public void login(JsonObject json, HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie, Settings settings) throws IOException {
+    public void login(JsonObject json, HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie, Settings settings) throws Exception {
         long start = System.currentTimeMillis();
         com.openexchange.guard.database.Access acc = new com.openexchange.guard.database.Access();
         response.setContentType("application/json");
@@ -565,15 +603,22 @@ public class Auth {
         response.addHeader("Access-Control-Allow-Origin", "*");
         response.setStatus(HttpServletResponse.SC_OK);
         response.addHeader("Connection", "close");
-        String lang = json.get("lang").getAsString();
+        String lang = Core.getStringFromJson(json, "lang");
         JsonObject returnJson = new JsonObject ();
         returnJson.add("lang", Languages.getJsonAvail(lang));
+        returnJson.addProperty("server", Config.serverVersion);
+        // Get public RSA keys for password encryption
+        String pubkey = Core.getPublicKey();
+        if (pubkey != null) {
+        	returnJson.addProperty("pubKey", pubkey);
+        }
+        
         int cid = 0;
         try {
 
-            int userid = json.get("user_id").getAsInt();
-            cid = json.get("cid").getAsInt();  
-            
+            int userid = Core.getIntFromJson(json, "user_id", true);
+            cid = Core.getIntFromJson(json, "cid", true);  
+            String email = json.get("email") == null ? null : json.get("email").getAsString();
             // Check lockout of bad attempts.  If excessive, don't give encrypted auth key
             if (CheckBad.isBad(Integer.toString(userid) + "-" + Integer.toString(cid), Config.bad_password_count)) {
             	returnJson.addProperty ("auth", "Lockout");
@@ -592,6 +637,24 @@ public class Auth {
             }
             returnJson.addProperty("cid", cid);
             GuardKeys keys = acc.getKeys(userid, cid);
+            
+            // Nothing found.  Previously a non-ox user and new to us?
+            if (keys == null) {
+            	if (email != null) {
+	            	RecipKey r = GetPubKey.checkMaster(email);  // check to see if we have a public non-ox key
+	            	if (r != null) {
+	            		if (r.cid < 0) {
+	            			try {
+	            				Upgrader.upgrade(r.userid, r.cid, userid, cid); // try to upgrade the account
+	            				keys = acc.getKeys(userid, cid); // try again
+	            			} catch (Exception ex) {
+	            				logger.error("Unable to upgrade email " + email);
+	            			}
+	            			
+	            		}
+	            	}
+            	}
+            }
             if (keys == null) {
             	returnJson.addProperty ("auth", "No Key");
             	returnJson.add ("settings", getSettings(null, settings));
@@ -619,7 +682,8 @@ public class Auth {
             try {
                 String oxpassword = json.get("ox_password") == null ? "" : json.get("ox_password").getAsString();
                 if (oxpassword.equals("null")) oxpassword = "";
-                String encrpass = json.get("encr_password").getAsString();
+                String encrpass = Core.getStringFromJson(json, "encr_password");
+                if (encrpass == null) encrpass = "";
                 String returndata = "Bad Password";
                 if (keys.getEncodedPrivate() == null) {
                     returndata = "NoPrivate";
@@ -636,23 +700,28 @@ public class Auth {
                         }
                     } else { // If no encr password, try the oxpassword
                         if (oxpassword.length() > 1) {
-                            if (keys.getDecodedPrivate(oxpassword, false) != null) {
-                                json.remove("encr_password");
-                                json.addProperty("encr_password", oxpassword); // The encryption password is our ox password
-                                returndata = encr.encryptAES(json.toString(), token);
-                                logger.debug("Auth with proper password for cid = " + cid + " userid = " + userid);
-                                // returndata = encr.encryptAES(json.toString(), token, cookie.JSESSIONID, Config.AESKeyLen);
-                            } else {
-                            	CheckBad.addBad(Integer.toString(userid) + "-" + Integer.toString(cid));  // Increment fail password counter
+                        	try {
+	                            if (keys.getDecodedPrivate(oxpassword, false) != null) {
+	                                json.remove("encr_password");
+	                                json.addProperty("encr_password", oxpassword); // The encryption password is our ox password
+	                                returndata = encr.encryptAES(json.toString(), token);
+	                                logger.debug("Auth with proper password for cid = " + cid + " userid = " + userid);
+	                                // returndata = encr.encryptAES(json.toString(), token, cookie.JSESSIONID, Config.AESKeyLen);
+	                            } else {
+	                            	CheckBad.addBad(Integer.toString(userid) + "-" + Integer.toString(cid));  // Increment fail password counter
+	                            	logger.debug("Auth, OG user, settings only for cid = " + cid + " userid = " + userid);
+	                            }
+                        	} catch (BadPasswordException bp) {
+                        		CheckBad.addBad(Integer.toString(userid) + "-" + Integer.toString(cid));  // Increment fail password counter
                             	logger.debug("Auth, OG user, settings only for cid = " + cid + " userid = " + userid);
-                            }
+                        	}
                         }
                     }
                 }
                 if (json.has("extrapass")) { // File does extra login to encrypt passwords prior to creating links
                 	String extrapass = json.get("extrapass").getAsString();
                 	if (!extrapass.equals("")) {
-                    	String epass = encr.encryptAES(extrapass, cookie.JSESSIONID);
+                    	String epass = encr.encryptAES(extrapass, cookie.JSESSIONID, "epass", Config.AESKeyLen);
                     	returnJson.addProperty("encrExtra", epass);
                 	}
                 }
@@ -728,6 +797,9 @@ public class Auth {
         if (!json.has("oxguard")) {
             json.add("oxguard", new JsonPrimitive(check(setting, Settings.permissions.MAIL)));
         }
+        if (Config.showStatus == false) {
+        	json.addProperty("status", false);
+        }
 
         return (json);
     }
