@@ -80,11 +80,13 @@ import com.google.gson.JsonArray;
 import com.google.gson.JsonObject;
 import com.google.gson.JsonParser;
 import com.google.gson.JsonPrimitive;
+import com.openexchange.guard.config.Config;
 import com.openexchange.guard.database.RetrievedItem;
 import com.openexchange.guard.encr.Crypto;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.EncryptedObj;
 import com.openexchange.guard.encr.GuardKeys;
+import com.openexchange.guard.exceptions.GuardMissingParameter;
 import com.openexchange.guard.file.Fileshare;
 import com.openexchange.guard.mailcreator.Attachment;
 import com.openexchange.guard.ox.Api;
@@ -102,19 +104,20 @@ public class FileHandler {
      * @param request
      * @param response
      * @throws IOException
+     * @throws GuardMissingParameter 
      */
-    public void downloadFormFromClient(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie, boolean update) throws IOException {
+    public void downloadFormFromClient(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie, boolean update) throws IOException, GuardMissingParameter {
         FileItemFactory fif = new DiskFileItemFactory();
         ServletFileUpload sfu = new ServletFileUpload(fif);
         String jsondata = "";
         String itemID = EncrLib.getUUID(); // One itemID for email and it's attachments
-        int userid = Integer.parseInt(request.getParameter("userid"));
-        int cid = Integer.parseInt(request.getParameter("cid"));
-        String folderid = request.getParameter("folder");
-        String version = request.getParameter("version"); // Version of file for updating
-        String id = request.getParameter("id"); // Id of file for updating
-        String sessionID = request.getParameter("session");
-        String auth = request.getParameter("auth");
+        int userid = Core.getIntParameter(request, "userid", true);
+        int cid = Core.getIntParameter(request, "cid", true);
+        int folderid = Core.getIntParameter(request, "folder", update); 
+        int version = Core.getIntParameter(request, "version", update); // Version of file for updating
+        int id = Core.getIntParameter(request, "id", update); // Id of file for updating
+        String sessionID = Core.getStringParameter(request, "session", true);
+        String auth = Core.getStringParameter(request, "auth");
 
         if (!Auth.checkPermission(cookie, request, com.openexchange.guard.server.Settings.permissions.FILE)) {
             logger.error("not authorized");
@@ -153,7 +156,7 @@ public class FileHandler {
                                 String filename = fi.getName();
                                 filename = filename + ".grd";
                                 attach.filename = filename;
-                                attach.type = fi.getContentType();
+                                attach.type = "grd/" + fi.getContentType().replace("/", "--");
                                 attachments.add(attach); // Add to our array of attachments
                                 out.close();
                                 input.close();
@@ -166,9 +169,9 @@ public class FileHandler {
                 if (update) {// If updating file, then will be first item, get key, encrypt, and update
                     RetrievedItem parentFile = getUpdateFile(
                         URLDecoder.decode(auth, "UTF-8"),
-                        Integer.parseInt(id),
-                        Integer.parseInt(folderid),
-                        Integer.parseInt(version),
+                        id,
+                        folderid,
+                        version,
                         request,
                         cookie,
                         sessionID);
@@ -189,7 +192,7 @@ public class FileHandler {
                         false,
                         attach.filename);
                     Api ap = new com.openexchange.guard.ox.Api(cookie, sessionID, request.getHeader("User-Agent"));
-                    String oxResp = ap.updateFile(attach, jsondata, Integer.parseInt(id), Integer.parseInt(folderid));
+                    String oxResp = ap.updateFile(attach, jsondata, id, folderid);
                     if (oxResp.toLowerCase().contains("error")) {
                         sendFail(response, oxResp);
 
@@ -267,21 +270,22 @@ public class FileHandler {
      * @param send
      * @return
      * @throws IOException
+     * @throws GuardMissingParameter 
      */
-    public Attachment getFile(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie, boolean send) throws IOException {
-        int itemid = Integer.parseInt(request.getParameter("id"));
-        int folder = Integer.parseInt(request.getParameter("folder"));
-        int version = Integer.parseInt(request.getParameter("version"));
-        int userid = Integer.parseInt(request.getParameter("userid"));
-        int cid = Integer.parseInt(request.getParameter("cid"));
-        String lang = request.getParameter("language");
-        String auth = request.getParameter("auth");
-        String filename = request.getParameter("filename");
-        String sessionID = request.getParameter("session");
-        String encrExtra = request.getParameter("encrextra");
+    public Attachment getFile(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie, boolean send) throws IOException, GuardMissingParameter {
+        int itemid = Core.getIntParameter(request, "id", true);
+        int folder = Core.getIntParameter(request, "folder", true);
+        int version = request.getParameter("version") == null ? -1 : Integer.parseInt(request.getParameter("version"));
+        int userid = Core.getIntParameter(request, "userid", true);
+        int cid = Core.getIntParameter(request, "cid", true);
+        String lang = Core.getStringParameter(request, "language");
+        String auth = Core.getStringParameter(request, "auth");
+        String filename = Core.getStringParameter(request, "filename", true);
+        String sessionID = Core.getStringParameter(request, "session", true);
+        String encrExtra = Core.getStringParameter(request, "encrextra");
         UserData data;
         if (auth.length() < 20) {
-            String password = request.getParameter("password");
+            String password = Core.getStringParameter(request, "password", true);
             data = new UserData();
             data.encr_password = password;
             data.userid = userid;
@@ -302,11 +306,11 @@ public class FileHandler {
                 return (null);
             }
             String file = crypto.decodeEncrObj(userid, cid, obj, data.encr_password);
-
+            boolean sizematch = (file.length() == obj.length) || (obj.length <= 0);
             if (file != null) {// Data was stored base64, so we need to decode when returned
                 if (file.startsWith("EXTRA")) { // Check for extra password
                     EncrLib encr = new EncrLib();
-                    String extra = encr.decryptAES(encrExtra, cookie.JSESSIONID);
+                    String extra = encr.decryptAES(encrExtra, cookie.JSESSIONID, "epass", Config.AESKeyLen);
                     file = encr.decryptAES(file.substring(5), extra, obj.ItemID, obj.keyLength);
                     if (file.length() < 2) {
                         sendOK(response, GetText.gettext("Incorrect password", lang));
@@ -321,6 +325,7 @@ public class FileHandler {
                     sendOK(response, GetText.gettext("Exceeded attempts, try again later", lang));
                     return (null);
                 }
+                obj.content_type = obj.content_type.replace("grd/", "").replace("--","/");
                 if (send) {
                     if ((request.getParameter("download") != null) && (request.getParameter("download")).contains("true")) {
                         saveToDownload(
@@ -335,6 +340,7 @@ public class FileHandler {
                     return (null);
                 }
                 Attachment decoded = new Attachment();
+                decoded.sizeMatch = sizematch;
                 decoded.filename = filename;
                 decoded.encrContent = Base64.decodeBase64(file);
                 decoded.type = obj.content_type;
@@ -412,15 +418,15 @@ public class FileHandler {
         }
     }
 
-    public void encryptFile(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) {
+    public void encryptFile(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws GuardMissingParameter {
 
-            int id = Integer.parseInt(request.getParameter("id"));
-            int folder = Integer.parseInt(request.getParameter("folder"));
-            int version = Integer.parseInt(request.getParameter("version"));
-            int userid = Integer.parseInt(request.getParameter("userid"));
-            int cid = Integer.parseInt(request.getParameter("cid"));
+            int id = Core.getIntParameter(request, "id", true);
+            int folder = Core.getIntParameter(request, "folder", true);
+            int version = Core.getIntParameter(request, "version", true);
+            int userid = Core.getIntParameter(request, "userid", true);
+            int cid = Core.getIntParameter(request, "cid", true);
         try {
-            String sessionID = request.getParameter("session");
+            String sessionID = Core.getStringParameter(request, "session", true);
 
             if (!Auth.checkPermission(cookie, request, permissions.MAIL)) {
                 logger.error("not authorized");
@@ -449,7 +455,7 @@ public class FileHandler {
             }
             Crypto crypt = new Crypto();
             String itemID = EncrLib.getUUID();
-            String newjson = getFileJson(itemID, folder, description, id, cid);
+            String newjson = getFileJson(itemID, folder, description, userid, cid);
             encrfile.encrContent = crypt.encrItem(userid, cid, null, Base64.encodeBase64String(filedata), 0, 2, "", itemID);
             if (encrfile.encrContent == null) {
                 sendFail(response, "Unable to encrypt file");
@@ -461,20 +467,20 @@ public class FileHandler {
             }
             ArrayList<Attachment> attachments = new ArrayList<Attachment>();
             encrfile.filename = encrfile.filename + ".grd";
+            encrfile.type = "grd/" + encrfile.type.replace("/", "--");
             attachments.add(encrfile);
 
             // Save to ox backend
             String params = "files?action=new&filename=" + URLEncoder.encode(encrfile.filename) + "&session=" + sessionID;
             String resp = ap.filePost(newjson, params, attachments);
             if (resp.contains("error")) {
-                sendFail(response, "Unable to save encrypted file");
+                sendFail(response, "Unable to save encrypted file " + Core.getErrorDescr(resp));
                 return;
             }
             if (resp.contains("timestamp")) {
                 ap.deleteItem(id, folder);
                 sendOK(response, "OK");
             }
-            System.out.println(resp);
         } catch (Exception ex) {
             logger.error("Error encrypting file", ex);
             sendFail(response, "Unable to encrypt");
@@ -511,17 +517,21 @@ public class FileHandler {
         return (null);
     }
 
-    public void removeEncryption(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) {
+    public void removeEncryption(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws GuardMissingParameter {
 
-    	int itemid = Integer.parseInt(request.getParameter("id"));
-    	int folder = Integer.parseInt(request.getParameter("folder"));
-    	int version = Integer.parseInt(request.getParameter("version"));
+    	int itemid = Core.getIntParameter(request, "id", true);
+    	int folder = Core.getIntParameter(request, "folder", true);
+    	int version = Core.getIntParameter(request, "version", true);
         try {
             Attachment decoded = getFile(request, response, cookie, false);
             if (decoded == null) {
             	sendFail(response, "Bad Password");
             	return;
             }
+            if (!decoded.sizeMatch) {
+            	sendFail(response, "Bad file size match");
+            	return;
+            }
             decoded.filename = decoded.filename.replace(".grd2", "").replace(".grd", "");
             String sessionID = request.getParameter("session");
             Api ap = new Api(cookie, request);
@@ -539,13 +549,13 @@ public class FileHandler {
             attachments.add(decoded);
             String resp = ap.filePost(newjson.toString(), params, attachments);
             if (resp.contains("error")) {
-                sendFail(response, "Unable to save decoded file");
+                sendFail(response, "Unable to save decoded file " + Core.getErrorDescr(resp));
                 return;
             } else {
             	//  OK.  We've decoded and sent.  Let's check the backend has a file the same size before deleting original
             	int newitemid = getID(resp);
             	if (newitemid == 0) { // Returns 0 if the id isn't there
-            		sendFail(response, "Unable to save decoded file");
+            		sendFail(response, "Unable to save decoded file, bad file size match");
                     return;
             	}
             	// Get the new file data json, which includes file size
@@ -580,10 +590,10 @@ public class FileHandler {
     	}
     }
 
-    public void getShares(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie oxCookie) {
-    	int userid = Integer.parseInt(request.getParameter("userid"));
-    	int cid = Integer.parseInt(request.getParameter("cid"));
-    	String itemID = request.getParameter("id");
+    public void getShares(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie oxCookie) throws GuardMissingParameter {
+    	int userid = Core.getIntParameter(request, "userid", true); 
+    	int cid = Core.getIntParameter(request, "cid", true);
+    	String itemID = Core.getStringParameter(request, "id", true);
         try {
             Fileshare fs = new Fileshare();
             JsonArray json = fs.getShares(itemID, userid, cid);
@@ -592,7 +602,6 @@ public class FileHandler {
             } else {
                 Gson gson = new Gson();
                 String json_string = gson.toJson(json);
-                System.out.println(json);
                 com.openexchange.guard.util.Core.sendOK(response, json_string);
             }
         } catch (Exception ex) {
@@ -605,7 +614,6 @@ public class FileHandler {
             Api ap = new Api(oxCookie, request);
             String data = ap.getFolder(Integer.parseInt(request.getParameter("id")));
             if (!data.contains("error")) {
-                System.out.println(data);
                 com.openexchange.guard.util.Core.sendOK(response, data);
                 return;
             }
@@ -658,7 +666,6 @@ public class FileHandler {
                 }
             }
             Gson gson = new Gson();
-            System.out.println(gson.toJson(permissions));
             JsonObject newdata = new JsonObject();
             newdata.add("permissions", permissions);
             String addresult = ap.updateFolder(id, gson.toJson(newdata));
@@ -668,7 +675,6 @@ public class FileHandler {
                 com.openexchange.guard.util.Core.sendFail(response, "error adding folder perm");
                 return;
             }
-            System.out.println();
             com.openexchange.guard.util.Core.sendOK(response, "OK");
             return;
         } catch (Exception ex) {
@@ -748,34 +754,37 @@ public class FileHandler {
         }
     }
 
-    public void addExtraPassword(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie oxCookie) {
+    public void addExtraPassword(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie oxCookie) throws GuardMissingParameter {
+        
+    	JsonObject rdata = Core.getJSON(request);
+        String auth = Core.getStringParameter(request, "auth");
+        String password = Core.getStringFromJson(rdata, "password");
+        if (password != null) {
+        	if (password.equals("undefined")) password = null;
+        }
+
+        if (auth != null) {
+            UserData data = new UserData(auth, oxCookie);
+            if (data.ox_password != null) {
+                password = data.ox_password;
+            }
+        }
+        if (password == null) {
+            com.openexchange.guard.util.Core.sendFail(response, "Fail");
+            return;
+        }
+        int userid = Core.getIntFromJson(rdata, "userid", true);
+        int cid = Core.getIntFromJson(rdata, "cid", true);
+        int id = Core.getIntFromJson(rdata, "id", true);
+        int folder = Core.getIntFromJson(rdata, "folder", true);
+        int version = Core.getIntFromJson(rdata, "version", true);
+        String sessionID = Core.getStringParameter(request, "session", true);
+        String extraPass = Core.getStringFromJson(rdata, "extrapass", true);
+        if ((extraPass == null) || (extraPass.length() < 3)) {
+            com.openexchange.guard.util.Core.sendFail(response, "Fail");
+            return;
+        }
         try {
-        	JsonObject rdata = Core.getJSON(request);
-            String auth = request.getParameter("auth");
-            String password = rdata.get("password").isJsonNull() ? null : rdata.get("password").getAsString();
-            if (password.equals("undefined"))
-                password = null;
-            if (auth != null) {
-                UserData data = new UserData(auth, oxCookie);
-                if (data.ox_password != null) {
-                    password = data.ox_password;
-                }
-            }
-            if (password == null) {
-                com.openexchange.guard.util.Core.sendFail(response, "Fail");
-                return;
-            }
-            int userid = rdata.get("userid").getAsInt();
-            int cid = rdata.get("cid").getAsInt();
-            int id = rdata.get("id").getAsInt();
-            int folder = rdata.get("folder").getAsInt();
-            int version = rdata.get("version").getAsInt();
-            String sessionID = request.getParameter("session");
-            String extraPass = rdata.get("extrapass").getAsString();
-            if ((extraPass == null) || (extraPass.length() < 3)) {
-                com.openexchange.guard.util.Core.sendFail(response, "Fail");
-                return;
-            }
             // Pull old encrypted file and it's json data
             Api ap = new Api(oxCookie, request);
             EncryptedObj file = ap.getInfoStore(id, folder, version);
@@ -795,7 +804,7 @@ public class FileHandler {
             String decoded = crypt.decodeEncrObj(userid, cid, file, password); // Get base64 encoded decoded item
             if (decoded != null) {
                 String itemID = EncrLib.getUUID();
-                String newjson = getFileJson(itemID, folder, description, id ,cid);
+                String newjson = getFileJson(itemID, folder, description, userid ,cid);
                 encrfile.encrContent = crypt.encrItem(userid, cid, null, crypt.addExtraPass(decoded, extraPass, itemID), 0, 2, "", itemID);
                 String params = "files?action=new&filename=" + URLEncoder.encode(encrfile.filename) + "&session=" + sessionID;
                 ArrayList<Attachment> attachments = new ArrayList<Attachment>();
@@ -810,6 +819,7 @@ public class FileHandler {
             com.openexchange.guard.util.Core.sendFail(response, "fail");
         } catch (Exception ex) {
             logger.error("Error adding extra password", ex);
+            Core.sendFail(response, "fail");
         }
     }
 }
