@@ -64,6 +64,8 @@ import com.openexchange.guard.util.CheckOx;
 import com.openexchange.guard.util.Core;
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.encr.OffsiteKeys;
+import com.openexchange.guard.exceptions.GuardMissingParameter;
+import com.openexchange.guard.mail.UploadGuest;
 import com.openexchange.guard.ox.InterOx;
 
 public class MainServlet extends HttpServlet {
@@ -89,6 +91,7 @@ public class MainServlet extends HttpServlet {
         }
         String command = request.getQueryString();
         String action = getAction(command);
+        logger.info("Action: " + action + " From IP: " + Core.getIP(request));
         com.openexchange.guard.server.OxCookie cookie = new OxCookie(request.getCookies());
 
         logger.debug("get action " + action);
@@ -97,13 +100,29 @@ public class MainServlet extends HttpServlet {
         case "/file":
             FileHandler fh2 = new FileHandler();
             if (action.equals("get")) {
-            	fh2.getFile(request, response, cookie, true);
+            	try {
+            		fh2.getFile(request, response, cookie, true);
+            	} catch (Exception e) {
+                	logger.error("Problem getting decrypting file");
+                	handleException (e, request, response);
+                }
+            		
             }
             if (action.equals("remove")) {
-                fh2.removeEncryption(request, response, cookie);
+            	try {
+            		fh2.removeEncryption(request, response, cookie);
+            	} catch (Exception e) {
+            		logger.error("Problem removing encryption");
+            		handleException (e, request, response);
+            	}
             }
             if (action.equals("getshare")) {
-                fh2.getShares(request, response, cookie);
+            	try {
+            		fh2.getShares(request, response, cookie);
+            	} catch (Exception e) {
+            		logger.error("Problem getting shares");
+            		handleException (e, request, response);
+            	}
             }
             if (action.equals("getfolder")) {
                 fh2.getfolder(request, response, cookie);
@@ -117,26 +136,45 @@ public class MainServlet extends HttpServlet {
             switch (action) {
            
             case ("getattach"):
-                mh.getEncrAttach(request, response, cookie);
+            	try {
+            		mh.getEncrAttach(request, response, cookie);
+            	} catch (Exception e) {
+                	logger.error("Problem getting encrypted attachment");
+                	handleException (e, request, response);
+                }
                 break;
             case ("getkey"):
-                String email = request.getParameter("email");
-                if (email != null) {
-                    try {
-                        mh.getRecipKey(request, response, email);
-                    } catch (Exception e) {
-                        response.sendError(500);
-                    }
+                
+                try {
+                    mh.getRecipKey(request, response);
+                } catch (Exception e) {
+                	logger.error("Problem getting key");
+                	handleException (e, request, response);
                 }
                 break;
             case ("count"):
-                mh.getReadCount(request, response);
+            	try {
+            		mh.getReadCount(request, response);
+            	} catch (Exception e) {
+            		logger.error("Problem getting count");
+                	handleException (e, request, response);
+                }
                 break;
             case ("retract"):
-                mh.retractEmail(request, response, cookie);
+            	try {
+            		mh.retractEmail(request, response, cookie);
+            	} catch (Exception e) {
+            		logger.error("Problem retracting email");
+                	handleException (e, request, response);
+                }
                 break;
             case ("upsell"):
-            	Upsell.getUpsell(request, response);
+            	try {
+            		Upsell.getUpsell(request, response);
+            	} catch (Exception e) {
+            		logger.error("Problem getting upsell");
+                	handleException (e, request, response);
+                }
             	break;
             }
 
@@ -160,14 +198,25 @@ public class MainServlet extends HttpServlet {
                 try {
                     gst.getEncrAttach(request, response, cookie);
                 } catch (Exception e) {
-                    response.sendError(500);
+                	logger.error("Error getting guest attachment ", e);
+                	handleException (e, request, response);
                 }
                 break;
             case ("getquestion"):
-                gst.getQuestion(request, response, cookie);
+            	try {	
+            		gst.getQuestion(request, response, cookie);
+            	 } catch (Exception e) {
+                 	logger.error("Error getting guest question for reset ", e);
+                 	handleException (e, request, response);
+                 }
                 break;
             case ("resetpassword"):
-                gst.resetPassword(request, response, cookie);
+            	try {
+            		gst.resetPassword(request, response, cookie);
+            	} catch (Exception e) {
+                 	logger.error("Error doing guest password reset ", e);
+                 	handleException (e, request, response);
+                 }
                 break;
             }
             break;
@@ -175,19 +224,44 @@ public class MainServlet extends HttpServlet {
             InterOx iox = new InterOx();
             switch (action) {
             case ("getkey"):
-                iox.getKey(request, response);
+            	try {
+            		iox.getKey(request, response);
+            	} catch (Exception e) {
+                 	logger.error("Error interox get Key ", e);
+                 	handleException (e, request, response);
+                 }
                 break;
             case ("getcount"):
-                iox.getReadCount(request, response);
+            	try {
+            		iox.getReadCount(request, response);
+            	} catch (Exception e) {
+                 	logger.error("Error interox get count ", e);
+                 	handleException (e, request, response);
+                 }
                 break;
             case ("retract"):
-                iox.retract(request, response);
+            	try {
+            		iox.retract(request, response);
+            	} catch (Exception e) {
+                 	logger.error("Error interox retract ", e);
+                 	handleException (e, request, response);
+                 }
                 break;
             case ("verify"):
-                iox.verifyToken(request, response);
+            	try {
+            		iox.verifyToken(request, response);
+            	} catch (Exception e) {
+                 	logger.error("Error interox verify ", e);
+                 	handleException (e, request, response);
+                 }
                 break;
             case ("remshare"):
-                iox.deleteshare(request, response);
+            	try {
+            		iox.deleteshare(request, response);
+            	} catch (Exception e) {
+                 	logger.error("Error interox remove share ", e);
+                 	handleException (e, request, response);
+                 }
             }
             break;
         case "/keys":
@@ -208,7 +282,7 @@ public class MainServlet extends HttpServlet {
                     auth.logout(request, response, cookie);
                 } catch (Exception e) {
                 	logger.error("Error logging in", e);
-                    response.sendError(500);
+                	handleException (e, request, response);
                 }
                 break;
 	        case ("reset"): 
@@ -216,7 +290,7 @@ public class MainServlet extends HttpServlet {
 	        		auth.resetPass(request, response, cookie);
 	        	} catch (Exception e) {
 	        		logger.error("Error resetting", e);
-	                response.sendError(500);
+	        		handleException (e, request, response);
 	            }
             	break;
 	        case ("secondary"):
@@ -224,7 +298,7 @@ public class MainServlet extends HttpServlet {
 	        		auth.getSecondaryEmail(request, response, cookie);
 	        	} catch (Exception e) {
 	        		logger.error("Error getting secondary e-mail", e);
-	                response.sendError(500);
+	        		handleException (e, request, response);
 	        	}
             }
             break;
@@ -239,7 +313,7 @@ public class MainServlet extends HttpServlet {
                     DemoClass demo = new DemoClass();
                     demo.Reset(request, response);
                 } catch (Exception e) {
-                    response.sendError(500);
+                	handleException (e, request, response);
                 }
                 break;
             }
@@ -295,13 +369,18 @@ public class MainServlet extends HttpServlet {
         logger.debug("url:" + url);
         logger.debug("put action: " + action);
         logger.debug("command: " + command);
+        logger.info("Action: " + action + " From IP: " + Core.getIP(request));
         switch (url) {
         case "/mail":
             MailHandler mh = new MailHandler(cookie, request);
             switch (action) {
             case ("saveattachment"):
-                mh.saveDecodedAttach(request, response, cookie);
-                System.out.println("save attach called");
+            	try {
+            		mh.saveDecodedAttach(request, response, cookie);
+            	} catch (Exception e) {
+            		logger.error("Problem with save attachment " , e);
+                	handleException (e, request, response);
+                }
                 break;
             }
             break;
@@ -336,6 +415,7 @@ public class MainServlet extends HttpServlet {
         logger.debug("url:" + url);
         logger.debug("post action: " + action);
         logger.debug("command: " + command);
+        logger.info("Action: " + action + " From IP: " + Core.getIP(request));
         switch (url) {
         case "/mail":
             // Call mail(action);
@@ -343,13 +423,28 @@ public class MainServlet extends HttpServlet {
             MailHandler mh = new MailHandler(cookie, request);
             switch (action) {
             case ("emailform"):
-                mh.incomingEmailForm(request, response);
+                try {
+					mh.incomingEmailForm(request, response);
+				} catch (Exception e2) {
+					// TODO Auto-generated catch block
+					logger.error("Incoming email fail " , e2);
+					handleException (e2, request, response);
+				}
                 break;
             case ("getmail"):
                 try {
                     mh.getEncrEmail(request, response, cookie);
                 } catch (Exception e) {
-                    response.sendError(500);
+                	logger.error("Error with get email ", e);
+                	handleException (e, request, response);
+                }
+                break;
+            case ("emaildraftform"):
+            	try {
+            		mh.incomingDraftForm(request, response);
+            	} catch (Exception e) {
+                	logger.error("Error with saving draft ", e);
+                	handleException (e, request, response);
                 }
                 break;
             }
@@ -359,10 +454,20 @@ public class MainServlet extends HttpServlet {
             FileHandler fh = new FileHandler();
             switch (action) {
             case ("encrypt"):
-                fh.encryptFile(request, response, cookie);
+            	try {
+            		fh.encryptFile(request, response, cookie);
+            	} catch (Exception e) {
+                	logger.error("Error encrypting file ", e);
+                	handleException (e, request, response);
+                }
                 break;
         	case ("addextra"): 
-                fh.addExtraPassword(request, response, cookie);
+        		try {
+        			fh.addExtraPassword(request, response, cookie);
+        		} catch (Exception e) {
+                	logger.error("Error adding extra password ", e);
+                	handleException (e, request, response);
+                }
             break;
             }
 
@@ -370,16 +475,31 @@ public class MainServlet extends HttpServlet {
             FileHandler fh3 = new FileHandler();
             switch (action) {
             case ("new"):
-                fh3.downloadFormFromClient(request, response, cookie, false);
+            	try {
+            		fh3.downloadFormFromClient(request, response, cookie, false);
+            	} catch (Exception e) {
+            		logger.error("Error encrypting new file", e);
+            		handleException (e, request, response);
+            	}
                 break;
             case ("update"):
-                fh3.downloadFormFromClient(request, response, cookie, true);
+            	try {
+            		fh3.downloadFormFromClient(request, response, cookie, true);
+            	} catch (Exception e) {
+            		logger.error("Error updating file", e);
+            		handleException (e, request, response);
+            	}
                 break;
             }
             break;
         case "/mailattach":
             FileHandler fh2 = new FileHandler();
-            fh2.downloadFormFromClient(request, response, cookie, false);
+            try {
+            	fh2.downloadFormFromClient(request, response, cookie, false);
+            } catch (Exception e) {
+        		logger.error("Error mail attach", e);
+        		handleException (e, request, response);
+        	}
             break;
         case "/guest":
             Guest guest = new Guest();
@@ -388,8 +508,21 @@ public class MainServlet extends HttpServlet {
                 guest.ChangePass(request, response, cookie);
                 break;
             case ("getmail"):
-                guest.GetEmail(request, response, cookie);
+            	try {
+            		guest.GetEmail(request, response, cookie);
+            	} catch (Exception e2) {
+					logger.error("Error getting guest email", e2);
+                	handleException (e2, request, response);
+				}
                 break;
+            case ("upload"):
+            	UploadGuest uploader = new UploadGuest();
+            	try {
+					uploader.incomingEmailForm(request, response, cookie);
+				} catch (Exception e2) {
+					logger.error("Error uploading for guest", e2);
+                	handleException (e2, request, response);
+				}
         	}
             break;
         case "/login":
@@ -400,14 +533,19 @@ public class MainServlet extends HttpServlet {
                 auth.login(request, response, cookie);
                 break;
             case ("guest"):
-                auth.guestlogin(request, response);
+            	try {
+            		auth.guestlogin(request, response);
+            	} catch (Exception e) {
+                	logger.error("Error changing password", e);
+                	handleException (e, request, response);
+                }
                 break;
             case ("changepass"):
                 try {
                     auth.changepass(request, response, cookie);
                 } catch (Exception e) {
                 	logger.error("Error changing password", e);
-                    response.sendError(500);
+                	handleException (e, request, response);
                 }
                 break;
             case ("oxpassreset"):
@@ -415,16 +553,15 @@ public class MainServlet extends HttpServlet {
                     auth.changepassFromOxRecovery(request, response, cookie);
                 } catch (Exception e) {
                 	logger.error("Error during password reset", e);
-                    response.sendError(500);
-                    e.printStackTrace();
+                	handleException (e, request, response);
                 }
                 break;
             case ("create"):
                 try {
                     auth.createKeys(request, response, cookie);
                 } catch (Exception e1) {
-                    response.sendError(500);
                     logger.error("Error creating keys", e1);
+                    handleException (e1, request, response);
                 }
 
                 break;
@@ -432,12 +569,26 @@ public class MainServlet extends HttpServlet {
             	try {
             		auth.changeSecondaryEmail(request, response, cookie);
             	} catch (Exception e) {
-                    response.sendError(500);
                     logger.error("Error changing secondary email", e);
+                    handleException (e, request, response);
                 }
             	break;
             }
             break;
+        case "/settings":
+        	GuardSettings guard_settings = new GuardSettings();
+        	switch (action) {
+        	case ("set"):
+        		try {
+					guard_settings.setSettings(request, response, cookie);
+				} catch (Exception e) {
+					logger.error("Unable to set settings " , e);
+					handleException (e, request, response);
+				}
+        	    break;
+        	}
+           break;
+        	
         case "/interox":
             InterOx iox = new InterOx();
             switch (action) {
@@ -461,6 +612,11 @@ public class MainServlet extends HttpServlet {
         logger.debug("POST action processed, time " + Core.gettime(start));
     }
 
+    private void handleException (Exception ex, HttpServletRequest request, HttpServletResponse response) {
+    	String url = request.getRequestURI();
+    	Core.sendError(response, ex.getMessage());
+    	logger.error("Fail with url: " + url, ex);
+    }
     /**
      * Parse action from command string
      * 
@@ -479,7 +635,7 @@ public class MainServlet extends HttpServlet {
                 action = action.substring(7);
             }
         } catch (Exception ex) {
-
+        	logger.error("Problem parsing command: " + command, ex);
         }
         return (action);
     }
